local shadowCasters = require "tables.shadowCasters"
local lg = love.graphics
local ShadowCaster = require("lib.classic"):extend()
local passes = require "rendering"

local lightingValues = require "tables.lightingValues"

function ShadowCaster:deleteShadowCaster()
  for i, v in ipairs(shadowCasters) do
    if v == self then
      table.remove(shadowCasters, i)
      break
    end
  end
  
  
  
end


function ShadowCaster:addShadowCaster()
  createAttribute(self.attributes, "shadowRed", 1)
  createAttribute(self.attributes, "shadowGreen", 1)
  createAttribute(self.attributes, "shadowBlue", 1.1)
  createAttribute(self.attributes, "castsShadow", true)
  createAttribute(self.attributes, "shadowIntensity", 0.6)
  createAttribute(self.attributes, "shadowLength", 400)
  createAttribute(self.attributes, "shadowOffset", 70)
  table.insert(shadowCasters, self)
end

local function wrap(x, max)
  return (x-1)%max + 1
end

local function generateShadowPolyGroup(poly, direction, shadowLength)
  direction = vector.normalize(direction)
  
  local moveVector = direction * shadowLength
  
  local group = {}

  
  for i = 1, #poly*0.5 do
    
    local index1 = 2*i - 1
    local index2 = 2*i
    local index3 = wrap(2*i + 1, #poly)
    local index4 = wrap(2*i + 2, #poly)
    
    local point1 = Vec2(poly[index1], poly[index2])
    local point2 = Vec2(poly[index3], poly[index4])
    
    
    local point3 = point2 + moveVector
    local point4 = point1 + moveVector

    
    local newQuad = 
    {
      point1.x, point1.y,
      point2.x, point2.y,
      point3.x, point3.y,
      point4.x, point4.y
      }
    
    table.insert(group, newQuad)
  end
  
  return group
end

local maxIntensity = 4
local ambientColour = {1,1,1}


function ShadowCaster:getShadowOffset()
  return self.attributes.shadowOffset or 70
end


function ShadowCaster:getShadowLength()
  return self.attributes.shadowLength - self:getShadowOffset() + 70
end

local MAX_INTENSITY = 4
function ShadowCaster:drawShadow(poly, offset)
  offset = offset or 0
  
  
  local direction = lightingValues:getDirection()
  --local direction = vector.normalize(-vector.angleToVector(math.rad(270)))
  local offsetVector = direction * offset
  
  local mode, alphamode  = lg.getBlendMode()
  lg.setBlendMode("replace", "premultiplied")
  
  local r = self.attributes.shadowRed
  local g = self.attributes.shadowGreen
  local b = self.attributes.shadowBlue
  local a = 0.5
  
  
  local ar, ag, ab = unpack(passes:getAmbient())
  
  r = ar * MAX_INTENSITY * (1-a) + r*a
  g = ag * MAX_INTENSITY * (1-a) + g*a
  b = ab * MAX_INTENSITY * (1-a) + b*a
  
  lg.setColor(r*self.attributes.shadowIntensity/maxIntensity, g*self.attributes.shadowIntensity/maxIntensity, b*self.attributes.shadowIntensity/maxIntensity, 1)
  --lg.setColor(1,1,1)
  
  local shadowLength = self:getShadowLength()
  lg.push()
  lg.translate(offsetVector.x*camera.zoom, offsetVector.y*camera.zoom)
  
  for i, quad in ipairs(generateShadowPolyGroup(poly, direction, shadowLength)) do
    blue.graphics.polygon("fill", quad)
  end
  
  if #poly >= 6 then
    blue.graphics.polygon("fill", poly)
  end
  
  lg.pop()
  
  lg.setBlendMode(mode, alphamode)
end


return ShadowCaster