DragonCore = PhysicsItem:extend()
DragonCore.type = "dragonCore"
function DragonCore:new(x, y, coreType)
  
  self.coreType = coreType
  
  local shape = love.physics.newCircleShape((self:getTexture():getWidth()*0.5 - 1)*smallTextureScale-0.5)
  
  DragonCore.super.new(self, x, y, shape)
  
  
  
  self.fixture:setFriction(0.1)
  
end


function DragonCore:getTexture(coreType)
  coreType = self.coreType or coreType
  if coreType == "green" then
    return texture.green_dragon_core
  elseif coreType == "red" then
    return texture.red_dragon_core
  elseif coreType == "purple" then
    return texture.purple_dragon_core
  else
    --Blue
    return texture.blue_dragon_core
  end
end


function DragonCore:getLightColour(coreType)
  coreType = coreType or self.coreType
  if coreType == "green" then
    return {0.5,1,0.4}
  elseif coreType == "red" then
    return {1,0.1,0.2}
  elseif coreType == "purple" then
    return {0.8,0.79,1}
  else
    --Blue
    return {0.5,0.7,1}
  end
end


function DragonCore:getLightIntensity(coreType)
  coreType = coreType or self.coreType
  if coreType == "green" then
    return 0.017
  elseif coreType == "red" then
    return 0.03
  elseif coreType == "purple" then
    return 0.07
  else
    --Blue
    return 0.015
  end
end


function DragonCore:postSolveCallback(a, b, coll, normalimpulse, tangentimpulse)
  
  normalimpulse = normalimpulse - 0.37
  if normalimpulse > 8 then
    local volume = math.min(1, normalimpulse/40)*0.15
    sound.effect.core_tap:setVolume(volume)
    sound.effect.core_tap:play()
  end
end

function DragonCore:getLightZ(coreType)
  coreType = coreType or self.coreType
  if coreType == "green" then
    return -6.5
  elseif coreType == "purple" then
    return -15.5
  else
    --Blue
    return -5
  end
end



function DragonCore:draw(x, y, r, coreType)
  --Optional values to be added so that the holder can draw it

  local texture = self:getTexture(coreType)
  
  r = r or self.body:getAngle()
  
  blue.graphics.setZ(0)
  love.graphics.setColor(1,1,1)
  
  if not x then
    x, y = self:getX(), self:getY()
  end 
  
  blue.graphics.drawCentredShaded(texture, x, y, r, smallTextureScale)
  
  
  addLight(
    x, 
    y, 
    self:getLightZ(coreType), -- Z
    self:getLightIntensity(coreType),
    self:getLightColour(coreType)
  )

end


function DragonCore:drawOverlay()
  if debugMode then
    love.graphics.setColor(1,1,1)
    local pos = self:getPosition()
    local x, y = pos.x, pos.y
    blue.graphics.circle("line", x, y, self.shape:getRadius())
  end
end