Fireball = require("entities.projectile"):extend()
Fireball.type = "fireball"
local waterSources = require "tables.waterSources"

local lightsFire = require "tables.lightsFire"


local LingerLight = require "entities.lingeringLight"

function Fireball:new(x, y, vx, vy)
  Fireball.super.new(self, x, y, vx, vy)
  --self.body:setLinearDamping(1)
  self.body:setMass(0.25)
  self.fixture:setUserData(self)
  self.life = 2
  self.sparkCountdown = 0
  self.setsFire = true
  self.brightness = 1
  self.trail = TrailRibbon()
  table.insert(lightsFire, self)
end


function Fireball:postSolveCallback(a, b, coll, normalimpulse, tangentimpulse)
  self.collided = true
end


function Fireball:explode()
  local vx, vy = self.body:getLinearVelocity()
  for i = 1, 25*settings.graphics.particleAmount do
    Spark(self.body:getX(),
      self.body:getY(), 
      0.5 + love.math.random()*2, 
      vx + love.math.randomNormal()*100, 
      vy + love.math.randomNormal()*200, 0)
  end
  self:delete()
end


function Fireball:getTailLength()
  return 50
end


function Fireball:updateTrail(dt)
  
  local pos = self:getPosition()
  local x, y = pos.x, pos.y
  
  self.trail:setPosition(x, y)
  
  local pointDensity = 10
  local length = self:getTailLength()
  local pointDistance = length/pointDensity
  
  local pointDistanceSqrd = self.trail:distanceToFirstPointSqrd()

  
  if (not pointDistanceSqrd or (pointDistanceSqrd > pointDistance^2)) then
    self.trail:addPoint(x, y)
  end

  
  self.trail:trimPoints(length)
end


function Fireball:update(dt)
  
  
  
  if not self.body:isDestroyed() then
    if self.collided then
      self:explode()
      return
    end
    
    if not self:effectedByTimeBubble({makeInactive = true}, self.shape:getRadius() --[[+ self:getTailLength()]]) then
      self.life = self.life - dt
      
      self.sparkCountdown = self.sparkCountdown - dt
      
      if self.life > 0 then
        while self.sparkCountdown <= 0 do
          local vx, vy = self.body:getLinearVelocity()
          Spark(self.body:getX(),self.body:getY(), 1, vx + love.math.random(-100, 99) + love.math.random(), vy + love.math.random(-100, 99) + love.math.random())
          self.sparkCountdown = self.sparkCountdown + (1/60)/settings.graphics.particleAmount
        end
        self:updateBrightnessAndTrailLights()
        self:updateTrail(dt)
      else
        
        self:explode()
        return
      end
      
      
      for i, source in ipairs(waterSources) do
        if source:checkInside(self.body:getPosition()) then
          
          local sound = sound.effect.extinguish:clone()
          
          sound:play()
          
          self:delete()
          return
        end
      end
      
      
    end
    
    
    
    
    
  end
end


function Fireball:delete()
  self.remove = true
  self:addLingerLight()
  self.body:destroy()
  
  
  for i, v in ipairs(lightsFire) do
    if v == self then
      table.remove(lightsFire, i)
      break
    end
  end
end


function Fireball:trailLightBehaviour(behindPolygon)
  --This function adds a linger light if the fireball disappears behind a polygon
  --Ew this code is a mess but I don't really care at this point

  if behindPolygon and not self.behindPolygonLastFrame then
    self:addLingerLight()
  end
  
  self.behindPolygonLastFrame = behindPolygon
end


function Fireball:updateBrightnessAndTrailLights()
  if not self.body:isDestroyed() then
    
    
    local behindPolygon = self:isBehindPolygon()
    self:trailLightBehaviour(behindPolygon)
    --Brightness should never be above 1
    self.brightness = 1
    
    if behindPolygon then
      self.brightness = self.brightness * (behindPolygon.currentTransparency or 0)
    end
    
    
  end
end


function Fireball:addLingerLight()
  LingerLight(self.body:getX(), self.body:getY(), -5, self.brightness, {1,0.5,0.2}, 10)
end


function Fireball:isBehindPolygon()
  
  for i, polygon in ipairs(polygonTable) do
    if not polygon.attributes.collidable and (not polygon.attributes.isBackground) then
      if polygon:checkInside(self.body:getPosition()) then
        return polygon
      end
    end
  end
  return false
  
end


function Fireball:drawTrail()
  if not self.frozen then
    blue.graphics.setZ(0)
    love.graphics.setColor(1,0.7,0.2, 0.3)
    local radius = self.shape:getRadius()*2
    self.trail:drawTriangle(self.trail:getPoints(), radius, self:getTailLength())
  end
end


function Fireball:draw()
  if not self.body:isDestroyed() then
    
    
    love.graphics.setColor(1, 1, 1)
    
    local rot = vector.getOrientation(vector.create(self.body:getLinearVelocity())) + math.rad(90)
    
    if rot ~= rot then
      rot = 0
    end
    
  
    
    addLight(self.body:getX(), self.body:getY(), -5, self.brightness, {1,0.5,0.2})
    
    
    blue.graphics.drawCentredShaded(texture.fireball, self.body:getX(), self.body:getY(), rot)
  end
end