Firefly = Entity:extend()
Firefly.type = "firefly"
fireflyTable = {}

--Class for fireflies

function Firefly:new(x, y)
  Firefly.super.new(self, x, y)
  
  self.velocity = Vec2(0, 0)
  self.caught = false
  self.caughtTimer = 0
  self.moveTimer = 0
  self.glowState = math.rad(love.math.random(360))
  table.insert(drawTable, self)
  table.insert(fireflyTable, {x = self.x, y = self.y, activeStage = self.activeStage})
end


function Firefly:delete()
  --Note - fireflies are not deleted when collected
  Firefly.super.delete(self)
  for i, v in ipairs(fireflyTable) do
    if self.x == v.x and self.y == v.y then
      table.remove(fireflyTable, i)
      break
    end
  end
end


function Firefly:updateBuilder()
  
  if builder.mode == "fireflies" then
    if love.keyboard.isDown("delete") then
      local deletionDistance = 25
      if vector.getMagnitudeSqrd(Vec2(mouse.worldX, mouse.worldY) - self) < deletionDistance^2 then
        self:delete()
      end
    end
  end
end

function Firefly:update(dt)
  
  if not self:effectedByTimeBubble() then
    local collectionRadius = 150
    local distanceVector = Vec2(player1.body:getPosition()) - self
    
    local magSqrd = vector.getMagnitudeSqrd(distanceVector)
    
    self.moveTimer = self.moveTimer + dt
    
    if self.caught then
      self.caughtTimer = self.caughtTimer + dt
    end
    
    if (player1.shape:getRadius())^2 > magSqrd then
      self:collect()
    elseif collectionRadius^2 > magSqrd and not self.caught then
      self.caught = true
    end
    
    if self.caught and (not self.remove) then
      local force = 1000*(self.caughtTimer)^2
      local step = vector.normalize(distanceVector) * force * dt
      
      
      if vector.getMagnitudeSqrd(step) > magSqrd then
        self:collect()
      else
        self.x = self.x + step.x
        self.y = self.y + step.y
      end
      
    end
  end
end


function Firefly:collect()
  self:delete()
  
  local sound = sound.effect.firefly_collect:clone()
  sound:play()

end


function Firefly:draw()
  local radius = 5
  local minTransp = 0.4
  
  local transparency = (((math.sin( math.rad(self.moveTimer + self.glowState)*200)+1)/2)*(1-minTransp)) + minTransp
  
  local offsetX = math.sin(self.moveTimer*(22/9) + self.glowState)*5
  local offsetY = math.sin(self.moveTimer + self.glowState)*10
  

  love.graphics.setColor(1, 1, 1)
  blue.graphics.drawCentredShaded(texture.firefly_body, self.x+offsetX, self.y+offsetY)
  love.graphics.setColor(1, 1, 1, transparency)
  blue.graphics.drawCentredShaded(texture.firefly, self.x+offsetX, self.y+offsetY)
  
  
  if debugMode then
    love.graphics.setColor(1, 1, 0)
    blue.graphics.circle("line", self.x, self.y, 1)
  end
end

return fireflyTable