Brazier = Interactable:extend()
Brazier.type = "brazier"

local lightsFire = require "tables.lightsFire"

function Brazier:new(x, y, attributes)
  Brazier.super.new(self, x, y, attributes)
  
  
  self.attributes.singleUse = nil
  createAttribute(self.attributes, "startsLit", false) 
  createAttribute(self.attributes, "litTriggerTarget", "")
  createAttribute(self.attributes, "extinguishTimer", 0)
  createAttribute(self.attributes, "triggerOnExtinguish", false)
  createAttribute(self.attributes, "destroysFireball", false)
  
  self.attributes.litTexture = nil
  self.attributes.unlitTexture = nil
  
  
  
  if self.attributes.startsLit then
    self:light(nil, true)
  else
    self:extingish(true)
  end
  
  self:setLightZone()
  
  self.width  = self:getTexture():getWidth() *smallTextureScale*self.attributes.scaleX
  self.height = self:getTexture():getHeight()*smallTextureScale*self.attributes.scaleY
  
  self.timeSinceExtingished = 1/0
  self.timeSinceIgnited = 1/0
end


function Brazier:setLightZone()
  self.lightZoneHeight = 30*smallTextureScale
  self.lightZoneWidth  = 30*smallTextureScale
end



function Brazier:getTexture()
  return texture.brazier
end



function Brazier:light(lighter, doNotTrigger)
  
  --Return true if not already lit, return false if so
  if self.lit then 
    return false
  else
    self.timeSinceIgnited = 0
    self.lit = true
    self.active = true
    if self.attributes.animation then
      self:setDrawMode("animation")
    end
    self.extinguishTimer = self.attributes.extinguishTimer
    
    if not doNotTrigger then
      for i, target in ipairs(nameIndex:getEntities(self.attributes.litTriggerTarget)) do
        if target.activate then
          target:activate(nil, 1)
        end
      end
    end
    
    if self.attributes.destroysFireball then
      if lighter.explode then
        lighter:explode()
      end
    end
    
    return true
  end

  

end


function Brazier:extingish(doNotTrigger)
  local oldLit = self.lit
  self.lit = false
  self.active = false
  self.extinguishTimer = 0
  self:setDrawMode("texture")

  if (not doNotTrigger) then
    if self.attributes.triggerOnExtinguish then
      if self.attributes.litTriggerTarget ~= "" and nameIndex.entities[self.attributes.litTriggerTarget] then
        for i, target in ipairs(nameIndex:getEntities(self.attributes.litTriggerTarget)) do
          if target.activate then
            target:activate(nil, -1)
          end
        end
      end
    end
  end

  self.timeSinceExtingished = 0
  
  --Return true if already unlit, return false if not
  return not oldLit
end



function Brazier:activate()
  player1.fire = true
end


function Brazier:update(dt)
  if not self.lit then
    self.timeSinceExtingished = (self.timeSinceExtingished or 0) + dt
  end
  if self.lit then
    self.timeSinceIgnited = (self.timeSinceIgnited or 0) + dt
  end
  
  for i, fireball in ipairs(lightsFire) do
    
    if self.x - self.lightZoneWidth*0.5 < fireball.body:getX() and fireball.body:getX() < self.x + self.lightZoneWidth*0.5 and
      self.y - self.lightZoneHeight*0.5 < fireball.body:getY() and fireball.body:getY() < self.y + self.lightZoneHeight*0.5 then
      self:light(fireball)
      break
    end
  end
 
 
  local frozen = false
  if self.attributes.extinguishTimer > 0 then
    
    frozen = self:effectedByTimeBubble()
    
    if not frozen then
      if self.extinguishTimer > 0 and self.extinguishTimer - dt < 0 then
        self:extingish()
      else
        self.extinguishTimer = math.max(0, self.extinguishTimer - dt)
      end
    end
  end
  
  if not frozen then
    self:updateAnimation(dt)
  end
end


function Brazier:drawOverlay()
  Brazier.super.drawOverlay(self)

  if debugMode or (self.selected and builderMode) then

    love.graphics.setLineWidth("3")
    love.graphics.setColor(1,0.7,0)
    blue.graphics.rectangle("line", self.x - self.lightZoneWidth/2, self.y - self.lightZoneHeight/2, self.lightZoneWidth, self.lightZoneHeight)
    love.graphics.setLineWidth("1")
  
  end
end

return Brazier