FireLantern = require "entities.interactables.brazier":extend()
FireLantern.type = "fireLantern"

function FireLantern:new(...)
  FireLantern.super.new(self, ...)
  self.attributes.animation = nil
  self.flickerOffset = love.math.random()*1000
  self.moving = 0
  self.flameAnim = AnimationPlayer(animation.fire_lantern_flame)
end


function FireLantern:setLightZone()
  self.lightZoneHeight = self.height*1.2
  self.lightZoneWidth  = self.width*1.2
end


function FireLantern:update(dt)
  FireLantern.super.update(self, dt)
  self.flameAnim:update(dt)
  self.moving = math.max(self.moving - dt, 0)
end


function FireLantern:getFlameColour()
  return {237/255, 175/255, 81/255}
end


local ANIMATION_LENGTH = 0.1

function FireLantern:extingish(...)
  FireLantern.super.extingish(self, ...)
  self.moving = ANIMATION_LENGTH
end


function FireLantern:light(...)
  if not self.lit then
    self.moving = ANIMATION_LENGTH
  end
  FireLantern.super.light(self, ...)
  
end


function FireLantern:getTexture()
  
  if self.moving and self.moving > 0 then
    return texture.fire_lantern_frame_moving
  end
  
  if self.lit then
    return texture.fire_lantern_frame_lit
  else
    return texture.fire_lantern_frame_dim
  end
end


function FireLantern:getCoreTexture()
  
  return texture.fire_lantern_core_orange
  
end


function FireLantern:draw()
  blue.graphics.setZ(40)
  love.graphics.setColor(1,1,1)
  blue.graphics.drawCentredShaded(texture.fire_lantern_core_orange, self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
  FireLantern.super.draw(self)
  
  local sts = smallTextureScale

  local opacity = 1 - self.timeSinceExtingished/(ANIMATION_LENGTH*2)
  local colour = self:getFlameColour()
  if self.lit or opacity > 0 then
    if self.lit then
      opacity = 1
    end
    
    
    love.graphics.setColor(colour[1], colour[2], colour[3], 0.35*opacity)
    self.flameAnim:play(self:getX(), self:getY(), 0, sts, sts, false)
    
    
    local lightFlicker = 0.05
    local offsetX = 0
    local offsetY = 0
    local intensity = 0.5*opacity
    local z = 5
    
    local flashLength = 0.3
    if self.timeSinceIgnited < flashLength and self.lit then
      intensity = intensity + intensity*(1 - self.timeSinceIgnited/flashLength)*2
    end
      
    
    local flickerAmount
    do
      local x = (love.timer.getTime() + self.flickerOffset)*10
      flickerAmount = (math.sin(x) + math.sin(4*x/9) + math.sin(x*2.2)*0.5 + math.sin(x*5.2)*0.25)*0.5*lightFlicker
    end
    
    
    
    addLight(
      self:getX() + offsetX, 
      self:getY() + offsetY, 
      z,
      intensity + flickerAmount,
      colour
    )
  else
    
    local lightFlicker = 0.05
    local offsetX = 0
    local offsetY = 0
    local intensity = 0.15
    local z = 10

    
    addLight(
      self:getX() + offsetX, 
      self:getY() + offsetY, 
      z,
      intensity,
      colour
    )
  end
  
end