Squeeze = require("lib.classic"):extend()
Squeeze.type = "squeeze"

function Squeeze:new(x, y, attributes)
  
  
  
  self.curve = love.math.newBezierCurve()
  if x then
    self:addPoint(x, y)
  end
  
  self.attributes = attributes or {}
  
  createAttribute(self.attributes, "minSpeed", 10)
  createAttribute(self.attributes, "requiresFacing", true)
  createAttribute(self.attributes, "interactRadius", 100)
  createAttribute(self.attributes, "speedEnterRadius", 20)
  
  
  createAttribute(self.attributes, "renderQuality", 2)
  
  
  
  table.insert(drawTable, self)
  
end


function Squeeze:delete()
  
  self.remove = true
  
end


function Squeeze:updateCurve()
  self.curve = love.math.newBezierCurve(self:getPointsTable()) 
end



function Squeeze:addPoint(x, y, i)
  --[[if i then
    table.insert(self.points, i, Vec2(x,y))
  else
    table.insert(self.points, Vec2(x,y))
  end]]

  
  if i then
    if i < 0 then
      i = i + self.curve:getControlPointCount() + 2
    else
      i = i + 1
    end
  else
    i = self.curve:getControlPointCount() + 1
  end
  
  self.curve:insertControlPoint(x, y, i)
  self:updateCurve()
end


function Squeeze:removePoint(i)

  i = i or self.curve:getControlPointCount()
  
  self.curve:removeControlPoint(i)
  self:updateCurve()
  
end


function Squeeze:movePoint(i, x, y)
  
  self:removePoint(i)
  self:addPoint(x, y, i-1)

end


function Squeeze:interact(playerPosition)
  if player1.wetness > 0 then
    self:startSqueeze()
  else
    cout("Player is not wet enough!")
  end
end


function Squeeze:updateCurve()
  self:updateLength()
  if self.curve:getControlPointCount() > 1 then
    self.derivativeCurve = self.curve:getDerivative()
  end
end





function Squeeze:updateLength()
  
  local length = 0
  
  if self.curve:getControlPointCount() > 1 then
    local bezierPoints = self.curve:render(self.attributes.renderQuality)
    
    for i = 3, #bezierPoints, 2 do

      local lastPoint = Vec2(bezierPoints[i-2], bezierPoints[i-1])
      local point = Vec2(bezierPoints[i], bezierPoints[i+1])
      
      length = length + vector.getMagnitude(lastPoint - point)
      
    end
  end
  
  self.length = length
  
  return length
  
end


function Squeeze:getLength()
  return self.length
end


function Squeeze:getPositionFromDistance(distance)
  local id = distance
  
  if distance < 0 then
    return nil
  end
  
  --[[local t = distance/self:getLength()
  
  if t >= 1 or t <= 0 then
    return nil
  end
  
  local dir = vector.normalize(Vec2(self.derivativeCurve:evaluate(t)))
  
  
  return Vec2(self.curve:evaluate(t)), dir]]
  
  
  local renderPoints = self.curve:render()
  
  
  for i = 3, #renderPoints, 2 do
    local point = Vec2(renderPoints[i], renderPoints[i+1])
    
    local lastPoint = Vec2(renderPoints[i-2], renderPoints[i-1])
    local length = vector.getMagnitude(lastPoint - point)
    
    if length < distance then
      distance = distance - length
    else
      
      return vector.interpolate(lastPoint, point, distance/length), vector.normalize(point - lastPoint)
    end
      
    
  end
  
--  return nil
  
end


function Squeeze:startSqueeze()
  if not player1.animationInfo.isSqueezing then
    local sideIsFirst
    do
      local point1 = Vec2(self.curve:getControlPoint(1))
      local pointN = Vec2(self.curve:getControlPoint(-1))
      local playerPos = Vec2(player1.body:getPosition())
      
      local displacement1 = point1 - playerPos
      local displacement2 = pointN - playerPos
      local distance1 = vector.getMagnitudeSqrd(displacement1) 
      local distance2 = vector.getMagnitudeSqrd(displacement2) 
      
      sideIsFirst = distance1 < distance2 
      
    end

    local speed = math.max(self.attributes.minSpeed*love.physics.getMeter(), vector.getMagnitude(velocityVector))
    
    if sideIsFirst then
      player1.squeezeDistance = 0
      player1.squeezeSpeed = speed
    else
      player1.squeezeDistance = self:getLength()
      player1.squeezeSpeed = -speed
    end
    player1.animationInfo.isSqueezing = true
    player1.sqeezeTrack = self
  end
end




local radius = 10

function Squeeze:checkSelect()
  
  local mousePos = Vec2(mouse.worldX, mouse.worldY)
  
  local point1 = Vec2(self.curve:getControlPoint(1))
  local pointN = Vec2(self.curve:getControlPoint(-1))
  
  
  return vector.getMagnitudeSqrd(mousePos - point1) < radius^2 or vector.getMagnitudeSqrd(mousePos - pointN) < radius^2
end


function Squeeze:isInteractable()
  
  return true
  
end


function Squeeze:getClosestPoint(x, y)
  
  local selectionDistanceSrd = radius^2

  local closestDistanceSqrd
  local closestPoint

  local pointTable = self.curve:render(0)

  for i, v in ipairs(pointTable) do

    if i%2 == 0 then
      -- If even, continue
    else
      local pointX = pointTable[i]
      local pointY = pointTable[i+1]

      local pointDistanceSqrd = (x - pointX)^2 + (y - pointY)^2


      if closestDistanceSqrd == nil or pointDistanceSqrd < closestDistanceSqrd then
        closestDistanceSqrd = pointDistanceSqrd
        closestPoint = (i+1)/2
      end
    end
  end



  if closestDistanceSqrd <= selectionDistanceSrd then
    return closestPoint
  else
    return nil
  end
  
end


function Squeeze:selectPoint(x, y)

  --returns the index of the closest vertex of a polygon to a given position

  if builder.selectedVertex then
    return builder.selectedVertex
  end
  
  
  local closestPoint = self:getClosestPoint(x, y)
  
  builder.selectedVertex = closestPoint or builder.selectedVertex

end



function Squeeze:updateBuilder(dt)

  
  if not (builder.selectedObject == self) then
    if love.keyboard.isDown("delete") then
      
      if self:checkSelect() then
        
        self:delete()
        
      end
    end
    
    
    
    
  else
    
    if clickedThisFrame.primary then
      self:selectPoint(mouse.worldX, mouse.worldY)
    else
      
    end
    
    
    if love.mouse.isDown(1) then
      if builder.selectedVertex then
        
        self:movePoint(builder.selectedVertex, mouse.worldX, mouse.worldY)
        
      end
    elseif love.keyboard.isDown("delete") then
      
      local closestPoint = self:getClosestPoint(mouse.worldX, mouse.worldY)
      
      if closestPoint then
        self:removePoint(closestPoint)
      end
      
    end
  
    
  end
end


function Squeeze:update(dt)
  
  if vector.getMagnitudeSqrd(velocityVector) > (self.attributes.minSpeed*love.physics.getMeter())^2 and player1.wetness > 0 then
    
    local playerPos = Vec2(player1.body:getPosition())
    
    local point1 = Vec2(self.curve:getControlPoint(1))
    local pointN = Vec2(self.curve:getControlPoint(-1))
    
    local displacement1 = point1 - playerPos
    local distance1 = vector.getMagnitudeSqrd(displacement1) 
    
    if distance1 < self.attributes.speedEnterRadius^2 then
      
      if vector.dotProduct(player1.directionVector, displacement1/math.sqrt(distance1)) > 0 then
        self:startSqueeze()
      end
      
    else
    
    
      local displacement2 = pointN - playerPos
      local distance2 = vector.getMagnitudeSqrd(displacement2) 
      if distance2 < self.attributes.speedEnterRadius^2 then
        
        if vector.dotProduct(player1.directionVector, displacement2/math.sqrt(distance2)) > 0 then
          self:startSqueeze()
        end
        
      end
    end
  end
  
end


function Squeeze:drawOverlay()
  
  local lg = love.graphics
  
  
  
  if debugMode or builderMode then
    lg.push()
    camera:applyTransformations()
    

    if not self.finalised then
      lg.setColor(0.5,0.5,1, 0.7)
    else
      lg.setColor(0.5,0.5,1)
    end
    
    
    local point1 = Vec2(self.curve:getControlPoint(1))
    local pointN = Vec2(self.curve:getControlPoint(-1))
    
    
    
    if builder.selectedObject == self then
      
      if self.curve:getControlPointCount() > 1 then
        lg.line(self.curve:render(0))
      end
      lg.setColor(1,1,1)
      for i = 1, self.curve:getControlPointCount() do
        local x, y = self.curve:getControlPoint(i)
        lg.circle("line", x, y, radius)
      end
    end
    
    lg.setLineWidth(1)
    lg.setColor(1,1,0)
    lg.circle("line", point1.x, point1.y, self.attributes.speedEnterRadius)
    lg.circle("line", pointN.x, pointN.y, self.attributes.speedEnterRadius)
    lg.setColor(1,0,0)
    lg.circle("line", point1.x, point1.y, self.attributes.interactRadius)
    lg.circle("line", pointN.x, pointN.y, self.attributes.interactRadius)
    
    
    
    
    
    if self.curve:getControlPointCount() > 1 then
      lg.setColor(0,1,0)
      lg.line(self.curve:render(self.attributes.renderQuality))
      
    end
    
    lg.pop()
  end
  
  
  
  Prop.drawInteractPrompt(self)
end