LevelPolygon = Entity:extend()
LevelPolygon.type = "levelPolygon"
LevelPolygon:implement(require("entities.components.castsShadows"))
polygonTable = {}


require "entities/polygonSpecial"

--Class for level polygons

function LevelPolygon:new(x, y, inputPoints, attributes, pixpol)
  
  Polygon.super.new(self, x, y)
  
  self.attributes = attributes or {}
  
  
  createAttribute(self.attributes, "isBackground", false)
  createAttribute(self.attributes, "draw", true)
  createAttribute(self.attributes, "depth", 5)
  createAttribute(self.attributes, "textureName", "stone")
  createAttribute(self.attributes, "smooth", false)
  createAttribute(self.attributes, "hasPhysics", false)
  createAttribute(self.attributes, "hasGravity", false)
  createAttribute(self.attributes, "springBack", true)
  createAttribute(self.attributes, "rotateBack", true)
  createAttribute(self.attributes, "protected", false)
  createAttribute(self.attributes, "superTexture", true)
  createAttribute(self.attributes, "collidable", true)
  createAttribute(self.attributes, "wetness", 0)
  createAttribute(self.attributes, "priorityDraw", false)
  createAttribute(self.attributes, "zoomCamera", 0)
  createAttribute(self.attributes, "fadeWhenBehind", false)
  createAttribute(self.attributes, "transparency", 0)
  createAttribute(self.attributes, "wrapMode", "repeat")
  
  
  if self.attributes.priorityDraw then
    self.attributes.drawPriority = self.attributes.drawPriority or 1
  else
    self.attributes.drawPriority = self.attributes.drawPriority or 0
  end
  
  self.attributes.priorityDraw = nil
  
  
  
  self.currentTransparency = self.attributes.transparency or 0
  

  self.attributes.ignore = false -- It's not recommended that you create an attribute manually, but I always want loaded polygons to not be ignored
  
  self.selected = false
  table.insert(drawTable, self)
  table.insert(polygonTable, self)
  
  
  if inputPoints then
    self.points = inputPoints
    self.finalised = true
    self:makeStatic(pixpol)
  else
    self.points = {}
    self.finalised = false
    self.points[1] = Vec2(x, y)
    self.numberOfPoints = 1
  end
  
  if self.type ~= "darkZone" then
    self:addShadowCaster()
  end
  
end



function LevelPolygon:addPoint(x, y)
  local point = Point(x, y)
  local lastPoint = self.points[#self.points]
  local firstPoint = self.points[1]


  --Limiter to prevent box2D error
  if point:distanceToTargetSqrd(lastPoint) > (0.005*64)^2 and point:distanceToTargetSqrd(firstPoint) > (0.005*64)^2 then
    table.insert(self.points, point)
    self.numberOfPoints = self.numberOfPoints + 1
    return point
  else
    if clickedThisFrame.primary then
      cout("Point was too close to an adjacent point!")
      local negativeSound = sound.effect.ui.negative:clone()
      negativeSound:play()
    end

    return lastPoint
  end
end

function LevelPolygon:removePoint(index)
  self.meshChanged = true
  if index then

    index = 2*index - 1

    table.remove(self.points, index)
    table.remove(self.points, index)

  else
    table.remove(self.points, self.numberOfPoints)
    self.numberOfPoints = self.numberOfPoints - 1
  end
end


function LevelPolygon:selectPoint(x, y, table)

  --returns the index of the closest vertex of a polygon to a given position

  if builder.selectedVertex then
    return builder.selectedVertex
  end


  local selectionDistance = 15/camera:getFinalZoom()
  local selectionDistanceSrd = selectionDistance^2

  local closestDistanceSqrd
  local closestPoint
  
  table = table or self.points

  for i, v in ipairs(table) do

    if i%2 == 0 then
      -- If even, continue
    else
      local pointX = table[i] + self.originalX
      local pointY = table[i+1] + self.originalY

      local pointDistanceSqrd = (x - pointX)^2 + (y - pointY)^2


      if closestDistanceSqrd == nil or pointDistanceSqrd < closestDistanceSqrd then
        closestDistanceSqrd = pointDistanceSqrd
        closestPoint = (i+1)/2
      end
    end
  end


  if closestDistanceSqrd <= selectionDistanceSrd then
    builder.selectedVertex = closestPoint
    return closestPoint
  else
    return nil
  end
end



function LevelPolygon:movePoint(x, y, i)
  -- i is the index of the vertex
  -- NOTE: polygons in this form are contained as a single table, odd values being the x and even values being the y.
  -- x = 2i - 1
  -- y = 2i
  self.meshChanged = true
  self.points[2*i - 1] = x
  self.points[2*i] = y
end


function LevelPolygon:finalise()
  self.finalised = true
  local newPointFormat = {}
  for i, point in ipairs(self.points) do
    table.insert(newPointFormat, point.x)
    table.insert(newPointFormat, point.y)
  end
  self.points = {}
  self.points = newPointFormat


  self:makeStatic()
  self:setAttributes()
end


function LevelPolygon:makeStatic(pixpol)
  if self.finalised then    
    
    self.body    = love.physics.newBody(world, 0, 0)
    self.shape   = love.physics.newChainShape(true, self.points)
    self.fixture = love.physics.newFixture(self.body, self.shape)
    self.fixture:setFriction(0.8)
    self.fixture:setRestitution(0)
    if self.attributes.draw then
      self:getPixpol(pixpol)
    end
  else
    cout("Cannot make static, polygon is not finalised!")
  end
end



function LevelPolygon:remake()

  self:setAttributes()
  if self.attributes.draw and not self.remove then
    if self.meshChanged then
      self:getPixpol()
      self.meshChanged = false
    else
      self.mesh:setTexture(texture[self.attributes.textureName])
    end
  end
end


function LevelPolygon:setAttributes()

  if not self.remove then
    if self.attributes.isBackground then
      self.sortType = "backgroundPolygon"
    else
      self.sortType = "levelPolygon"
    end
    
    
    
    if self.attributes.hasPhysics then
      local newCentre = self:getTopCorner()


      self.shape = nil
      self.body:destroy()
      self.body = love.physics.newBody(world, newCentre.x, newCentre.y, "dynamic")

      for i=1, #self.points, 2 do
        self.points[i] = self.points[i]
        self.points[i+1] = self.points[i+1]
      end

      local triangles = love.math.triangulate(self.points)
      self.triangleShapes = {}
      self.triangleFixtures = {}
      for i, triangle in ipairs(triangles) do
        self.triangleShapes[i] = love.physics.newPolygonShape(unpack(triangle))
        self.triangleFixtures[i] = love.physics.newFixture(self.body, self.triangleShapes[i])
        self.triangleFixtures[i]:setFriction(0.8)
        self.triangleFixtures[i]:setRestitution(0)
      end

      self.body:setLinearDamping(1)
      self.body:setAngularDamping(1)

      if self.originalX then
        self.body:setPosition(self.originalX, self.originalY)
      else
        self.originalX = newCentre.x
        self.originalY = newCentre.y
      end

      self.originalCentre = {}
      self.originalCentre.x, self.originalCentre.y = self.body:getWorldCenter()
      self:getStickTracks()
      
    elseif not (self.attributes.isBackground or not self.attributes.collidable) then
      if not self.fixture:isDestroyed() then
        self.fixture:destroy()
      end
      self.shape = love.physics.newChainShape(true, self.points)
      self.fixture = love.physics.newFixture(self.body, self.shape)
      self.fixture:setRestitution(0)
      self.fixture:setFriction(0.8)
      
      --Get the stick tracks if it's collidable 
      self:getStickTracks()
      
    else
      if not self.fixture:isDestroyed() then
        self.fixture:destroy()
      end
    end

    if self.attributes.hasGravity then
      self.body:setGravityScale(1)
    else
      self.body:setGravityScale(0)
    end
  end

  self.centrePos = self:computeCentrePos()
  self.cullRadius = self:getCullRadius()
end


function LevelPolygon:treatCoordinates(x, y)
  --Snap to an axis or any other changes I wish to make

  if love.keyboard.isScancodeDown("lshift") then
    local lastPoint = self.points[self.numberOfPoints]

    local roundResolution = 50 --math.floor(50/camera.zoom*100)/100

    x = math.floor((x-lastPoint.x % roundResolution)/roundResolution+0.5)*roundResolution + lastPoint.x % roundResolution
    y = math.floor((y-lastPoint.y % roundResolution)/roundResolution+0.5)*roundResolution + lastPoint.y % roundResolution



  end
  return x, y
end


function LevelPolygon:delete()

  if not self.finalised then
    builder.activePolygon.remove = true
    builder.activePolygon = nil
    
  elseif not self.remove then
    
    if builder.selectedPolygon == self then
      builder.selectedPolygon = nil
    end
    if not self.attributes.isBackground then
      if not self.fixture:isDestroyed() then
        self.fixture:destroy() 
      end
    end
    self.body:destroy()
    self.remove = true
    
    
    for i, polygon in ipairs(polygonTable) do
      if polygon == self then
        table.remove(polygonTable, i)
      end
    end


    if self.mesh then
      self.mesh:release()
    end
    flushDrawTable()
  end
  self:deleteShadowCaster()
end


function LevelPolygon:getTopCorner()
  local lowestX = self.points[1]
  local lowestY = self.points[2]


  for i, coord in ipairs(self.points) do
    if i > 2 then
      if i%2 == 1 then
        --X
        if coord < lowestX then
          lowestX = coord
        end
      else
        --Y
        if coord < lowestY then
          lowestY = coord
        end
      end
    end
  end
  return Vec2(lowestX, lowestY)
end



function LevelPolygon:getStickTracks()

  self.stickTracks = {}
  
  
  local function getExtrudePoint(ax, ay, bx, by, cx, cy, direction)
    --b is the point extruding from
    --order of points is a b c
    --p is the point that is being found
    
    local a = Vec2(ax, ay)
    local b = Vec2(bx, by)
    local c = Vec2(cx, cy)
    
    local ab = vector.normalize(b - a)
    local bc = vector.normalize(c - b)
    
    --Extra distance to add to the player radius
    local buffer = 0.5
    
    
    local abPerp = vector.turn90Degrees(ab) * (player1.shape:getRadius()+buffer)*direction
    
    if vector.isParallel(ab, bc) then
      
      local extrudePoint = b + abPerp
      
      return extrudePoint.x, extrudePoint.y
    else
    
      local bcPerp = vector.turn90Degrees(bc) * (player1.shape:getRadius()+buffer)*direction
      
      
      local p1 = a + abPerp
      local p2 = p1 + ab
      local p3 = b + bcPerp
      local p4 = p3 + bc
      
      
      local returnX, returnY = line.infiniteIntersection(p1.x, p1.y, p2.x, p2.y, p3.x, p3.y, p4.x, p4.y)
      
      
      
      return returnX, returnY
    end
    
  end
  
  local signedArea = getPolygonArea(self.points)
  local direction = signedArea/math.abs(signedArea)
  
  local worldPoints = self.body:getWorldPoints(unpack(self.points))
  
  for i = 1, #self.points, 2 do
    local ax = self.points[i]
    local ay = self.points[i+1]
    
    local bx = self.points[(i+1) % #self.points + 1]
    local by = self.points[(i+2) % #self.points + 1]
    
    local cx = self.points[(i+3) % #self.points + 1]
    local cy = self.points[(i+4) % #self.points + 1]
    
    
    
    local pointX, pointY = getExtrudePoint(ax, ay, bx, by, cx, cy, direction)
    
    
    --Don't add if the points are parallel. pointX will be false if parallel
    if pointX then
      table.insert(self.stickTracks, pointX)
      table.insert(self.stickTracks, pointY)
    end
    
  end
  
  if self.attributes.hasPhysics then
    self.untranslatedStickTracks = copyTable(self.stickTracks)
  end
end



function LevelPolygon:averagePoints()
  local average = Vec2(0, 0)
  for i = 1, #self.points, 2 do
    local point = Vec2(self.points[i], self.points[i+1])

    average = point + average

  end
  return average / (#self.points/2)
end


function LevelPolygon:checkSelect()
  return (not self.attributes.ignore) and self:checkInside(mouse.worldX, mouse.worldY)
end


function LevelPolygon:checkInside(x, y)
  if self.finalised then
    if self.body then
      return isPointInPolygon(x, y, {self.body:getWorldPoints(unpack(self.points))})
    else
      return isPointInPolygon(x, y, self.points)
    end
  else
    return false
  end
end


function LevelPolygon:findMiddle(i)
  --Return point exactly between two points

  local numberOfPoints = 0

  for i, v in ipairs(self.points) do
    numberOfPoints = numberOfPoints +1
  end
  numberOfPoints = numberOfPoints/2


  local p1 = Vec2(self.points[2*i-1], self.points[2*i])
  local p2 = {}
  if i == numberOfPoints then
    p2 = Vec2(self.points[1], self.points[2])
  else
    p2 = Vec2(self.points[2*(i+1)-1], self.points[2*(i+1)])
  end


  local returnVector = (p1 + p2) / 2

  return returnVector.x, returnVector.y
end

local function wrap(x, max)
  return (x-1)%max + 1
end


function LevelPolygon:computeCentrePos()
  if self.body and self.body:getType( ) ~= "static" then
    return Vec2(self.body:getWorldCenter())
  else
    local ax = 0
    local ay = 0
    local totalWeight = 0
    for i = 1, #self.points*0.5 do
      
      
      local index1 = 2*i - 1
      local index2 = 2*i
      local index3 = wrap(2*i + 1, #self.points)
      local index4 = wrap(2*i + 2, #self.points)
      
      local x1 = self.points[index1]
      local y1 = self.points[index2]
      local x2 = self.points[index3]
      local y2 = self.points[index4]
      
      local weight = math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
      
      local x = (x1 + x2)*0.5
      local y = (y1 + y2)*0.5
      
      totalWeight = totalWeight + weight
      ax = ax + x*weight
      ay = ay + y*weight
    end
    
    ax = ax/totalWeight
    ay = ay/totalWeight
    
    return Vec2(ax, ay)
  end
end


function LevelPolygon:getCentrePos() 
  
  if self.body and self.body:getType() ~= "static"  then
    return self.body:getWorldCenter()
  else
    if not self.centrePos then
      self.centrePos = self:computeCentrePos()
    end
    return self.centrePos.x, self.centrePos.y
  end
end


function LevelPolygon:getCullRadius()
  local cx, cy
  if self.body then
    cx, cy = self.body:getLocalPoint(self:getCentrePos())
  else
    cx, cy = self:getCentrePos()
  end
  local furthestX
  local furthestY
  
  local furthestDistance = 10
  
  for i = 1, #self.points*0.5 do
    local x = self.points[i*2-1]
    local y = self.points[i*2]
    local distanceSq = (cx - x)^2 + (cy - y)^2
    
    if distanceSq > furthestDistance then
      furthestX = x
      furthestY = y
      furthestDistance = distanceSq
    end
    
  end
  
  
  return math.sqrt(furthestDistance) + smallTextureScale, furthestX, furthestY
end


local function rectIntersection(a, b)
  
  local ax1 = a.x - a.width*0.5
  local ax2 = a.x + a.width*0.5
  local ay1 = a.y - a.height*0.5
  local ay2 = a.y + a.height*0.5
  
  local bx1 = b.x - b.width*0.5
  local bx2 = b.x + b.width*0.5
  local by1 = b.y - b.height*0.5
  local by2 = b.y + b.height*0.5
  
  --https://discord.com/channels/329400828920070144/474705430434807819/1241069490045845545
  
  
  local x_condition = (ax1-bx2)*(bx1-ax2)
  local y_condition = (ay1-by2)*(by1-ay2)
  
  return math.min(x_condition, y_condition) > 0
  
end


function LevelPolygon:shouldDraw()
  
  if self.attributes.draw == false then
    return false
  end

  if not self.centrePos then
    self.centrePos = self:computeCentrePos()
  end
  
  local shadowLength = self:getShadowLength() + self:getShadowOffset()
  
  local shadowOffsetX = shadowLength
  local shadowOffsetY = shadowLength
  
  local polyBox = {}
  polyBox.x, polyBox.y = self:getCentrePos()
  self.cullRadius = self.cullRadius or self:getCullRadius()
  polyBox.width  = self.cullRadius*2 + shadowOffsetX*2
  polyBox.height = self.cullRadius*2 + shadowOffsetY*2
  
  local screenBox = {}
  screenBox.x = camera.x
  screenBox.y = camera.y
  screenBox.width, screenBox.height = camera:getViewDimentions()
  
  return rectIntersection(polyBox, screenBox)

end



function LevelPolygon:update(dt)

  if self.attributes.hasPhysics then
    
    if self.untranslatedStickTracks then
      self.stickTracks = {self.body:getWorldPoints(unpack(self.untranslatedStickTracks))}
    end
    
    if not self:effectedByTimeBubble({useMassCentre = true}) then

      if self.attributes.springBack then

        local centreX, centreY = self:getCentrePos()
        local f = Vec2(self.originalCentre.x - centreX, self.originalCentre.y - centreY)
        self.body:applyForce(f.x*self.body:getMass(), f.y*self.body:getMass())
        
      end

      if self.attributes.rotateBack then

        local torque = -self.body:getAngle()*self.body:getMass()*10000
        self.body:applyTorque(torque)
        
      end
    end
  end
  
  
  if self.attributes.wetness > 0 and self:checkInside(player1.body:getPosition()) then
    
    if player1.wetness + dt ~= self.attributes.wetness then
      player1:enterWater(self)
    end
    player1.wetness = self.attributes.wetness
  end
  
  
  if self.attributes.fadeWhenBehind then
    local fadeSpeed = 4
    if (not self.body:isDestroyed()) and self:checkInside(player1.body:getPosition()) then
      
      self.currentTransparency = self.currentTransparency + dt * fadeSpeed
      
    else
      
      self.currentTransparency = self.currentTransparency - dt * fadeSpeed
      
    end
    self.currentTransparency = math.max(math.min(self.currentTransparency, 1), 0)
  end
  
  
end



function LevelPolygon:updateBuilder()

  if builder.mode == "polygon" then
    if self.finalised then

      if self.selected then
        if love.keyboard.isScancodeDown("delete") then
          local selectedPoint = self:selectPoint(mouse.worldX, mouse.worldY, self.points)
          if selectedPoint then
            self:removePoint(selectedPoint)
          end
        end
      end
      
      local inThisPolygon
      if self.body and (not self.body:isDestroyed()) then
        inThisPolygon = self:checkInside(mouse.worldX, mouse.worldY)
      end
      
      if inThisPolygon then
        --If mouse is over polygon
        if love.keyboard.isScancodeDown("delete") and not self.selected then
          if (not self.attributes.protected) or (love.keyboard.isDown("end") or love.keyboard.isDown("backspace")) then
            self:delete()
          end
        end
      end
      

      if self.selected then
        self:modifyPoints(inThisPolygon)
      end
    end
  end

  if self.selected then
    --Selected but not finalised
    builder.selectedPolygon = self
  elseif builder.selectedPolygon == self then
    builder.selectedPolygon = nil
  end
end



function LevelPolygon:modifyPoints(inThisPolygon)

  if love.mouse.isDown(1) then

    local selectedPoint = self:selectPoint(mouse.worldX, mouse.worldY)
    if selectedPoint then
      --Selected and finalised
      local transformedX, transformedY = lockToPixels(mouse.worldX, mouse.worldY, "small")
      self:movePoint(transformedX - self.originalX, transformedY - self.originalY, selectedPoint)
    else
      if clickedThisFrame.primary then
        --Add a point to a polygon
        local appendPointTable = {}
        for i = 1, #self.points, 2 do
          local x, y = self:findMiddle((i+1)/2)
          table.insert(appendPointTable, x)
          table.insert(appendPointTable, y)
        end
        
        local selectedAppendPoint = self:selectPoint(mouse.worldX, mouse.worldY, appendPointTable)
        if selectedAppendPoint then
          local i = 2*selectedAppendPoint-1
          local appendX = appendPointTable[i]
          local appendY = appendPointTable[i+1]

          table.insert(self.points, i+2, appendX)
          table.insert(self.points, i+3, appendY)
          --Fixes weird move vertex bug
          builder.selectedVertex = selectedAppendPoint+1

        end
      end
    end
    
    if clickedThisFrame.primary and (not selectedPoint) and love.keyboard.isScancodeDown("lshift") and not self.attributes.protected then
      self.grabbed = true
      builder.polyGrabOffset = {}
      for i = 1, #self.points, 2 do
        builder.polyGrabOffset[i] = mouse.worldX - self.points[i]
        builder.polyGrabOffset[i+1] = mouse.worldY - self.points[i+1]
      end
    end
  else
    builder.polyGrabOffset = {}
    self.grabbed = false
  end
  
  
  if self.grabbed then
    for i = 1, #self.points/2 do     
      local x, y = lockToPixels(mouse.worldX - builder.polyGrabOffset[2*i - 1], mouse.worldY - builder.polyGrabOffset[2*i])
      self:movePoint(x, y, i)
    end
  end
  
end


function LevelPolygon:getShadowOffset()
  local offset = self.attributes.shadowOffset or 70
  if self.attributes.isBackground then
    offset = offset*6
  end
  return offset
end



local componentDrawShadow = LevelPolygon.drawShadow
function LevelPolygon:drawShadow()
  
  if self.attributes.castsShadow then
    
    local offset = self:getShadowOffset()
    
    if self.finalised then
      
      if self:shouldDraw() and self.type ~= "darkZone" then
        if self.finalised then
          if self.body then
            componentDrawShadow(self, {self.body:getWorldPoints(unpack(self.points))}, offset)
          else
            componentDrawShadow(self, self.points, offset)
          end
        end
      end
    end
    
  end
end


function LevelPolygon:draw()
  if self.finalised or builderMode then
    if self.finalised then
      if self:shouldDraw() then
        if texture[self.attributes.textureName].main then
          texture[self.attributes.textureName].main:setWrap(self.attributes.wrapMode)
        else
          texture[self.attributes.textureName]:setWrap(self.attributes.wrapMode)
        end
        self:drawFinalised()
      end
      
    end
    
  end
end



function LevelPolygon:drawOverlay()
  
  if self.finalised then

    local worldPoints
    if self.body then
      worldPoints = {self.body:getWorldPoints(unpack(self.points))}
    end
    
    if self.selected and builderMode then
      love.graphics.setColor(0,0,1)
      
      if worldPoints then
        blue.graphics.polygon("line", worldPoints)
      else
        blue.graphics.polygon("line", self.points)
      end
    end
    if debugMode or (self.selected and builderMode) then
      for i = 1, #self.points, 2 do
        if i == 1 then
          love.graphics.setColor(0,0,0)
        else
          love.graphics.setColor(1,0,0)
        end

        self.originalX = self.originalX or 0
        self.originalY = self.originalY or 0

        blue.graphics.circle("line", self.points[i] + self.originalX, self.points[i+1] + self.originalY, 2/camera:getFinalZoom())
        if self.selected and builderMode then
          love.graphics.setColor(1,0,0, 0.3)
          blue.graphics.circle("line", self.points[i] + self.originalX, self.points[i+1] + self.originalY, 15/camera:getFinalZoom())
        end
      end



      if self.triangleShapes and self.triangleShapes[1] then
        for i, triangle in ipairs(self.triangleShapes) do

          local shapePoints = {self.body:getWorldPoints(triangle:getPoints())}
          for i = 1, #shapePoints, 2 do
            if i == 1 then
              love.graphics.setColor(0,0,0)
            else
              love.graphics.setColor(0,1,1)
            end
            blue.graphics.circle("line", shapePoints[i], shapePoints[i+1], 5/camera:getFinalZoom())

          end
        end
      end



      if self.selected and builderMode then
        for i = 1, #self.points, 2 do
          local x, y = self:findMiddle((i+1)/2)
          love.graphics.setColor(0,1,0)
          x = x + self.originalX
          y = y + self.originalY
          blue.graphics.circle("line", x, y, 2/camera:getFinalZoom())
        end
      end
    end

    if debugMode then
      if not self.attributes.draw then
        love.graphics.setColor(1,0,0)
        if worldPoints then
          blue.graphics.polygon("line", worldPoints)
        else
          blue.graphics.polygon("line", self.points)
        end
      end

      
      

      
      if self.body then
        local centreX, centreY
        love.graphics.setColor(1,0,0)
        blue.graphics.circle("fill", self.body:getX(), self.body:getY(), 4/camera:getFinalZoom())
        centreX, centreY = self.body:getWorldCenter()
        love.graphics.setColor(0,1,0)
        blue.graphics.circle("fill", centreX, centreY, 4/camera:getFinalZoom())
      end
      --Draw cull circle
      local x, y = self:getCentrePos()
      if x and self.cullRadius then
        love.graphics.setColor(1,0,1, 0.5)
        blue.graphics.circle("line", x, y, self:getCullRadius())
      end
    end
  else
    local lastPoint = false
    local firstPoint = false

    for i, point in ipairs(self.points) do
      love.graphics.setColor(0,0,0)
      blue.graphics.circle("line", point.x, point.y, 3/camera:getFinalZoom())
      if i ~= 1 then
        love.graphics.setColor(0.8,0.2,0.2)
        blue.graphics.line(point.x, point.y, lastPoint.x, lastPoint.y)
      else
        firstPoint = point
      end
      lastPoint = point
    end
    if self.numberOfPoints > 2 then
      love.graphics.setColor(0.2,0.2,1,0.2)
      blue.graphics.line(lastPoint.x, lastPoint.y, firstPoint.x, firstPoint.y)
    end

    lastPoint = self.points[self.numberOfPoints]
    local mouseX, mouseY = self:treatCoordinates(mouse.worldX, mouse.worldY)

    love.graphics.setColor(0,0,1,0.5)
    blue.graphics.line(mouseX, mouseY, lastPoint.x, lastPoint.y)
  end
  
  --[[
  
  if debugMode and self.stickTracks then
    love.graphics.setColor(1,1,1)
    blue.graphics.linePolygon(self.stickTracks)
    
    for i = 1, #self.stickTracks, 2 do
      
      local x = self.stickTracks[i]
      local y = self.stickTracks[i+ 1]
      
 
      
      blue.graphics.circle("line", x, y, 5)
      blue.graphics.print((i+1)*0.5, x, y)
    end
  
  end
  ]]
  
  
end

