local lastjoystickX = 0
local lastjoystickY = 0
local function getFlightControlDirection()
  
  if gamepadMode then
    
    local safezoneRadius = 0.5
    
    if vector.getMagnitudeSqrd(flightJoystick) > safezoneRadius^2 then
      local direction = vector.normalize(flightJoystick)
      lastjoystickX = direction.x
      lastjoystickY = direction.y
      
      return direction
    else
      
      return Vec2(lastjoystickX, lastjoystickY)
    end
  else
    return vector.normalize(player1:getMouseRelativeToPlayer())
  end
end


function Player:setTempMaxSpeed(speed)
  
  self.tempMaxSpeed = speed
end


function Player:getDefaultMaxSpeed(settings)
  settings = settings or {}
  return (settings.minDragSpeed or 30) * love.physics.getMeter()
end


function Player:getMaxSpeed(settings)
  local absoluteMaxSpeed = 100 * love.physics.getMeter()
  
  return math.min(math.max(self:getDefaultMaxSpeed(settings), self.tempMaxSpeed or 0), absoluteMaxSpeed) 
end



function Player:handleMaxSpeed(dt, settings)
  --Break max speed if above it
  local normalMaxSpeed = self:getDefaultMaxSpeed(settings)
  
  local velSq = vector.getMagnitudeSqrd(self:getVelocity())
  
  
  
  if velSq > (normalMaxSpeed*1.1)^2 then
    local isFlying = self.flightMode == "swooping" and (not self.tucked)
    
    if (not self.tempMaxSpeed) or (not isFlying) or (self:getMaxSpeed(settings)*0.9)^2 > velSq then
      self:setTempMaxSpeed(math.sqrt(velSq))
    end
  else
    self:setTempMaxSpeed()
  end
  
  --[[
  if self.tempMaxSpeed then
    self.tempMaxSpeed = self.tempMaxSpeed - dt * love.physics.getMeter() * 0.5
    
  end]]

end


function Player:isBoundryBreaking()
  local isFlying = self.flightMode == "swooping" and (not self.tucked)
  return self.tempMaxSpeed and isFlying
end


function Player:applyAerodynamics(dt, settings)
  
  self:handleMaxSpeed(dt, settings)
  
  
  settings = settings or {}
  local minDragSpeed = self:getMaxSpeed(settings)
  tester = minDragSpeed / love.physics.getMeter()
  local linearDrag = (settings.linearDrag or 1) * 0.2
  local wingDrag = (settings.wingDrag or 1) * 0.004
    
  local normal = vector.normalize(vector.turn90Degrees(self.directionVector))
  local velocityNormal = vector.normalize(vector.turn90Degrees(self:getVelocity()))
  
  --Profile of the line from the velocity vector's position
  local areaVector = vector.project(self.directionVector, velocityNormal)
  --The normal, but with the direction the right direction
  local correctedNormal = vector.normalize(vector.project(self:getVelocity(), normal))
  
  --Final drag vector
  self.dragVector = correctedNormal * -wingDrag * vector.getMagnitude(areaVector) * vector.getMagnitudeSqrd(self:getVelocity())
  self.body:applyForce(self.dragVector.x, self.dragVector.y)
  
  --Apply linear drag
  if vector.getMagnitudeSqrd(self:getVelocity()) >= minDragSpeed^2 then
    
    local vx, vy = self.body:getLinearVelocity()
    
    self.body:applyForce(
      -vx * linearDrag, 
      -vy * linearDrag)
  end
end

function Player:smoothRotation(dt, speed)
  local controlDirection = getFlightControlDirection()
  local rotationSpeed = speed

  
  local mouseAngle = vector.getOrientation(controlDirection)
  local directionAngle = vector.getOrientation(self.directionVector)
  --Degrees/second
  local rotationStep = math.rad(rotationSpeed)*dt

  --If the difference in angle is less than a rotation step, just set angle to the mouse
  if math.abs(mouseAngle - directionAngle) <= rotationStep then
    self.directionVector = vector.normalize(controlDirection)


  elseif vector.angleIsClockwise(self.directionVector, controlDirection) then      
    --Clockwise
    self.directionVector = vector.angleToVector(directionAngle + rotationStep) 
  else 
    --Anticlockwise 
    self.directionVector = vector.angleToVector(directionAngle - rotationStep) 
  end
end



function Player:applyWingFlapForce(dt)
  local flapRatio = 0.3
    
  if ((1-flapRatio)*self.flapCooldown <= self.timeOfLastFlap) and self.tucked == false then
    
    
    local force
    
    if self.inWater then
     --[[ --Flap harder if moving slowly
      local maxBonusForce = 600
      local speedSqrd = vector.getMagnitudeSqrd(self:getVelocity())
      
      local forceBonus = 1/(speedSqrd*0.0000001 + (1/maxBonusForce))
      local forceMult = 140 + forceBonus
      force = forceMult/flapRatio]]
      
    else
      --Flap harder if moving slowly
      local maxBonusForce = 300
      local speedSqrd = vector.getMagnitudeSqrd(self:getVelocity())
      
      local forceBonus = 1/(speedSqrd*0.0000001 + (1/maxBonusForce))
      local forceMult = 140 + forceBonus
      force = forceMult/flapRatio
    end

    
  
    --Just to help get unstuck
    if self:isStuck() then
      force = force * 10
    end
    
    self.body:applyForce(self.directionVector.x * force, self.directionVector.y * force)
  else
    self.flapping = false
  end
end



function Player:flyingBehaviour(dt)
  
  self.timeOfLastFlap = self.timeOfLastFlap - dt
  
  self.isFlapCooldown = self.timeOfLastFlap >= 0
  
  if not (self.animationInfo.isSqueezing or self.inWater) then
    if controls.flap.isDown and not self:isBoundryBreaking() then
      if not self.tucked then
        --TODO - Contain all the cooldown stuff in flap(), maybe also add a cooldown override argument
        if not self.isFlapCooldown then
          --Set flap timer to current time
          if self.flightMode == "hovering" or self.flightMode == "walking" then
            self.flightMode = "swooping"
            self.directionVector = getFlightControlDirection()
            if self.directionVector.x == 0 and self.directionVector.y == 0 then
              self.directionVector = Vec2(1,0)
            end
          end
          self:flap()
        end
      end
    end
  
    -- Smooth rotation ==================
    

    if self.flightMode == "swooping" and not self.tucked and not controls.lockDirection.isDown then
      self:smoothRotation(dt, math.max(540 / (vector.getMagnitude(self:getVelocity())*0.002+1), 120))
    end

    -- Aerodynamics area for swooping mode ============================================
    if self.tucked == false and self.flightMode == "swooping" then
      
      self:applyAerodynamics(dt)
      
    elseif self.tucked == false and self.flightMode == "hovering" then
      self:walkingBehaviour()
    end

    --If moving slow enough, don't bounce
    if vector.getMagnitudeSqrd(self:getVelocity()) < (10*64)^2 and not self.tucked then
      self.fixture:setRestitution(0)
    else
      self.fixture:setRestitution(0.3)
    end


    --Flap wings if valid

    self:applyWingFlapForce(dt)

    if self.tucked == false and self:canWalk() and not self.flapping then
      self.flightMode = "walking"
    elseif self.flightMode == "swooping" and math.deg(vector.angleBetween(self.directionVector, self:getVelocity())) > 120 and not self.flapping and vector.getMagnitude(self:getVelocity())/64 > 3 and (not self.tucked) and not self.stasis  then
      if controls.flap.isDown then
      
        local vx, vy = self.body:getLinearVelocity()

        self.body:applyForce(-vx, -vy)
      else
        --Stall if moving too fast backwards
        self.flightMode = "walking"
      end
    end
  end
end


function Player:airWalkingBehaviour()
  if not self:canWalk() then
    local maxForce = 25

    if controls.walkRight.isDown and controls.walkLeft.isDown then
      --Do nothing
    elseif controls.walkRight.isDown then
      self.body:applyForce(maxForce, 0)

    elseif controls.walkLeft.isDown then
      self.body:applyForce(-maxForce, 0)
    end
  end
end


function Player:blowbackBehaviour(dt)
  

  if settings.game.flightAssist then
  
    if self.flightMode == "swooping" and not self.tucked then
      
      local minimumSpeed = 18 * love.physics.getMeter()
      local minimumDistance = 500
      
      local atSpeed = minimumSpeed^2 <= vector.getMagnitudeSqrd(self:getVelocity())
      --local withinDistance = minimumDistance^2 >= vector.getMagnitudeSqrd(self.blowbackVector)
      
      
      if atSpeed then
        
        local rayLength = minimumDistance
        

        --I'm going to sort by distance by only looking at one axis, since it's on a 
        --straight line the distance can be compared with only one value with no need
        --for pythagoras, as long as the axis changes (which will be determined at the 
        --start). This might be overoptimising.
        
        local xSortMode
        
        local castStartX = self.body:getX()
        local castStartY = self.body:getY()
        local castEndX   = castStartX + self.directionVector.x * rayLength
        local castEndY   = castStartY + self.directionVector.y * rayLength
        
        local closestHit
        local closestHitNormal
        local closestDistance 
        
        local function setClosestHit(x, y, xn, yn)
          closestHit       = Vec2(x, y)
          closestHitNormal = Vec2(xn, yn)
          if xSortMode then
            closestDistance = math.abs(castStartX - x)
          else
            closestDistance = math.abs(castStartY - y)
          end
        end
        
        
        
        
        local function callback(fixture, x, y, xn, yn, fraction)
            --Set the sort mode and first values
            
          local userData = fixture:getUserData()
          
          --Exclude fixtures that can't be collided with
          if fixture:getCategory() == 2 then
            return 1
          end

          

          if xSortMode == nil then
          
            xSortMode = math.abs(castStartX - x) > math.abs(castStartY - y)
            setClosestHit(x, y, xn, yn)
            
          else
            
            if xSortMode then
              if closestDistance > math.abs(castStartX - x) then
                setClosestHit(x, y, xn, yn)
              end
            else
              if closestDistance > math.abs(castStartY - y) then
                setClosestHit(x, y, xn, yn)
              end
            end
            
          end
          return 1
        end
        
        world:rayCast(castStartX, castStartY, castEndX, castEndY, callback)
        
        --Closest point aquired
        self.closestPoint       = closestHit
        self.closestPointNormal = closestHitNormal
        
        local allignmentThreshold = 0
        
        if self.closestPoint then
          local allignment = math.max(vector.dot(-self.closestPointNormal, self.directionVector), 0.2)
          
          
          if allignment > allignmentThreshold then
            
            local distanceMult = minimumDistance/(vector.getMagnitude(Vec2(self.body:getPosition()) - self.closestPoint))-0.5
            
            local rotationSpeed = 2400 * allignment * distanceMult
            
            
            
            local rotationStep = math.rad(rotationSpeed)*dt
            local directionAngle = vector.getOrientation(self.directionVector)
            
            
            if vector.angleIsClockwise(-self.closestPointNormal, self.directionVector) then      
              --Clockwise

              self.directionVector = vector.angleToVector(directionAngle + rotationStep) 
            else 
              --Anticlockwise 

              self.directionVector = vector.angleToVector(directionAngle - rotationStep) 
            end
            
            
            --Going to use momentum based rotation
            
           --[[ if vector.angleIsClockwise(-self.closestPointNormal, self.directionVector) then      
              --Clockwise
              self.correctionRotationVelocity = math.max(math.abs(self.correctionRotationVelocity),  1200*allignment)
            else 
              --Anticlockwise 
              self.correctionRotationVelocity = -math.max(math.abs(self.correctionRotationVelocity),  1200*allignment)
            end]]
            

          end
          
        end
        
      end
    end
  end
end


local function hoverFlight(g, F, M)
  -- I'm gonna hate myself later for this
  -- Given known vector G, known unit vector F and a known magnitude of the resultant force M.
  -- vector M (which is unknown) must be at full length, and rests on the end of g, and the other on vector tF. tF (t being the magnitude of F) is of unknown length, but the direction is known.

  -- Using pythagorous and the quadratic equation, we can find t

  --      If you're not me in the future and reading this, sorry - I did most of it on paper

  local a = g.x
  local b = g.y
  local c = F.x
  local d = F.y

  local m = a*c + b*d
  local n = c^2 + d^2

  --Time for the quadratic equation :_)

  if 2*n == 0 then
    return 0
  end
  solution = (-(2*m) + math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)

  if solution < 0 then
    solution = (-(2*m) - math.sqrt((2*m)^2 - 4*n*(a^2 + b^2 - M^2))) / (2*n)
  end
  return solution
  --IT WORKS AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
end



function Player:hoverControls()
  self.dragVector = vector.normalize(self:getVelocity()) * -0.4*vector.getMagnitude(self:getVelocity())
  if not love.keyboard.isScancodeDown("down", "s") then
    self.body:applyForce(0, -gravity * self.body:getMass())
  end
  self.body:applyForce(self.dragVector.x, self.dragVector.y)

  local hoverVector = Vec(0)

  if controls.walkRight.isDown then
    hoverVector = hoverVector + Vec2(1, 0)
  end
  if controls.walkLeft.isDown then
    hoverVector = hoverVector + Vec2(-1, 0)
  end
  if love.keyboard.isScancodeDown("up", "w") and not love.keyboard.isScancodeDown("down", "s") then
    hoverVector = hoverVector + Vec2(0, -1)
  end

  hoverVector = vector.normalize(hoverVector)
  local maxForce = 120 + gravity * self.body:getMass()
  local wingForce = hoverFlight(Vec2(0, -gravity * self.body:getMass()), hoverVector, maxForce)
  hoverVector = hoverVector * wingForce

  self.body:applyForce(hoverVector.x, hoverVector.y)
  self.directionVector = vector.normalize(hoverVector + Vec2(0, -gravity*0.5 * self.body:getMass()))
end


function Player:hoveringBehaviour()
  if not self.tucked then
    if controls.hover.thisFrame then

      self.flightMode = toggle(self.flightMode, "hovering", "swooping")
      if self.flightMode == "swooping" then
        self.directionVector = vector.normalize(self:getMouseRelativeToPlayer())
        if self.directionVector.x == 0 and self.directionVector.y == 0 then
          self.directionVector = Vec2(1,0)
        end
      end
      self:untuck()
    end
    if self.flightMode == "hovering" then
      self:hoverControls()
    end
  end
end

function Player:tuckingBehaviour(dt)

  if self.bonked > 0 then
    self:tuck(true)
    self.bonked = self.bonked - dt
  end
  self.bonkImmunity = self.bonkImmunity - dt

  if self.tucked and not self.animationInfo.isSqueezing then
    self.directionVector = vector.angleToVector(self.body:getAngle()) 
    if self.contactWithGround and self.bonked <= 0 then
      if controls.walkLeft.isDown then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*-5)
        self.body:applyTorque(-400)
      end
      if controls.walkRight.isDown then
        --self.body:setAngularVelocity(self.body:getAngularVelocity()+dt*5)
        self.body:applyTorque(400)
      end
    end
    
    if controls.flap.thisFrame and self.bonked <= 0 then
      self:rollJump(dt)
    end
  end


  if controls.tuck.thisFrame or self.bonked > 0 then
    self:tuck()
  elseif controls.tuck.upThisFrame or (self.bonked <= 0 and not controls.tuck.isDown ) then
    self:untuck()
  end 

  --Setting angular velocity instead of fixing rotation makes it so that if you bonk you will roll properly
  --[[if not self.tucked then
    cout(self.body:getAngularVelocity())
    self.body:setAngularVelocity(self.body:getAngularVelocity())
    --self.body:setAngularVelocity(0) 
  end]]
end



function Player:flap()

  sound.effect.wingBeat:setVolume(1)
  sound.effect.wingBeat:play()
  self:stopSleeping()
  self.animationInfo.walking = false
  self.flightMode = "swooping"
  --TODO - change flapping to use dt instead of time
  self.timeOfLastFlap = self.flapCooldown
  self.flapping = true
end



function Player:rollJump(dt, forceJump)

  if self.contactWithGround or self.parried > 0 or love.timer.getTime() - self.animationInfo.timeOfLastContact <= 0.15 or forceJump then
    sound.effect.wingBeat:play()

    -- Set bonk immunity so we don't bump on the ground
    self.bonkImmunity = 0.1
    self.parried = 0
    
    self:resetParryWindow()
    
    local contactNormal = self.trueNormalWithGround
    
    if contactNormal == Vec2(0) then
      contactNormal = vector.normalize(velocityVector)
    else
      self.lastContactNormal = contactNormal
    end
    
    local direction
    local controlDirection = getFlightControlDirection()
    if contactNormal then
      if vector.dotProduct(contactNormal, controlDirection) > 0 then
        direction = controlDirection
      else
        
        local mirrorLine = vector.turn90Degrees(contactNormal)
        direction = vector.normalize((controlDirection - 2 * vector.dotProduct(controlDirection, mirrorLine) * mirrorLine)*-1)
        
      end
      
      

      --Redirect the previous velocity
      local boostAmount = 200
      local speed = vector.getMagnitude(self:getVelocity())*0.70 + boostAmount
      local newVelocity = direction*speed
      
      self.flightMode = "walking"
      
      self.body:setLinearVelocity(newVelocity.x, newVelocity.y)
      
      TempAnim(self.body:getX(), self.body:getY(), vector.getOrientation(direction)+math.rad(90), animation.effect_magicRipple)
    end
  end
end



function Player:tuck(noRotationBoost)
  
  self:stopSleeping()
  if not self.tucked and not self.stasis then
    self.timeTucked = 0
    --self.fixture:setFriction(0.2)
    self.tucked = true
    self.body:setAngle(vector.getOrientation(self.directionVector))



    if not noRotationBoost then
      local newRotationSpeed = self.body:getLinearVelocity()/self.shape:getRadius() * 1.2

      if controls.walkLeft.isDown and not controls.walkRight.isDown then
        newRotationSpeed = -math.min(10, math.abs(newRotationSpeed))
      elseif controls.walkRight.isDown then
        newRotationSpeed = math.min(10, math.abs(newRotationSpeed))
      else
        newRotationSpeed = math.max(math.min(10, newRotationSpeed), -10)
      end

      self.animationInfo.walking = false

      self.body:setFixedRotation(false)
      self.body:setAngularVelocity(newRotationSpeed)
    end
  end
end

function Player:untuck()
  
  if self.tucked then
    self.fixture:setFriction(self.normalFiction)
    self.tucked = false
    
    if self.flightMode == "swooping" or self.inWater then
      self.directionVector = vector.normalize(getFlightControlDirection())
    else
      self.directionVector = Vec2(self.lookDirection,0)
    end
    self.parried = 0
    
    self.body:setFixedRotation(true)
    self:resetParryWindow()
  end
end


function Player:canWalk()
  local maxAngle = math.rad(60)
  if self.contactWithGround then
    return math.acos(vector.dotProduct(self.normalWithGround, Vec2(0,-1))) <= maxAngle
  else 
    return false
  end
end


function Player:canWalkPart(normal)
  local maxAngle = math.rad(60)
  return math.acos(vector.dotProduct(normal, Vec2(0,-1))) <= maxAngle
end



function Player:walkingBehaviour()
  if self.stasisCountdown <= 0 and self:canWalk() and not self.animationInfo.isSqueezing then
    
    

    
    
    
    --Need to get relative tangential velocity without losing the sign
    --Project on to x axis, then normalize. Then multiply the resulting vector by the magnitude 
    
    self.falling = false
    
    local horizontalTangentVector = vector.project(self.tangentVelocity, Vec2(1,0))
    horizontalTangentVector = horizontalTangentVector * vector.getMagnitude(self.tangentVelocity)
    local relativeGroundVelocity = vector.project(self:getVelocity(), self.tangentVelocity) * 0.00002
    local contactWithGround = ((180 - math.deg(vector.angleBetween(self.normalWithGround, Vec2(0,-1)))*2))/180
    --Fraction based on the slope the ground is on
    

    local directionForce = vector.normalize(self.tangentWithGround) -- Vector
    
    
    if  (controls.walkLeft.isDown and controls.walkRight.isDown) or 
    not (controls.walkLeft.isDown or  controls.walkRight.isDown) then
      --If pressing both left and right, or pressing neither
      self.animationInfo.walking = false

      local groundVelocitySqrd = vector.getMagnitudeSqrd(relativeGroundVelocity)

      if groundVelocitySqrd >= 1100^2 then
        self.animationInfo.sliding = true
      elseif groundVelocitySqrd <= 10^2 then
        self.animationInfo.sliding = false
      end
      
    else

      self:stopSleeping()
      self.animationInfo.walking = true
      self.animationInfo.sliding = false


      if controls.walkLeft.isDown then
        
        self.lookDirection = -1
        local walkForce = 1000 * contactWithGround
        
        
        --directionForce should already be normalized here
        
        directionForce = (directionForce * walkForce) - relativeGroundVelocity

        if directionForce.x < -walkForce or directionForce.x > walkForce then
          directionForce = vector.normalize(directionForce) * walkForce
        end
      elseif controls.walkRight.isDown then

        self.lookDirection = 1
        local walkForce = - 1000 * contactWithGround

        --directionForce should already be normalized here
        
        directionForce = (directionForce*walkForce) - relativeGroundVelocity
        
        --TODO work out what's going on here
        if directionForce.x < -walkForce or directionForce.x > walkForce then
          directionForce = vector.normalize(directionForce) * -walkForce
        end
      end
    end
    
    self.directionVector = -vector.turn90Degrees(self.normalWithGround or Vec2(0,-1)) * self.lookDirection

    self.body:applyForce(directionForce.x, directionForce.y)
    
    if self.inWater then
      local linearDrag = 5
      local vx, vy = self.body:getLinearVelocity()
    
      self.body:applyForce(
        -vx * linearDrag, 
        -vy * linearDrag)
        
    end
    
    
  elseif love.timer.getTime() - self.animationInfo.timeOfLastContact > 0.2 then
    self.animationInfo.walking = false
  end
end


