Player = Entity:extend()
Player.type = "player"
local PARRY_COLOUR = {1, 211/255, 162/255}
local LingerLight = require "entities.lingeringLight"

--Class for the player character

function Player:new(x, y)
  Player.super.new(self, x, y)
  self.activeStage = nil --Make the player always active
  table.insert(drawTable, self)
  self.directionVector = Vec2(1, 0) 
  
  self.character = "Drea"

  self.body = love.physics.newBody(world, x, y, "dynamic")
  self.body:setSleepingAllowed(false)

  
  -- the player's shape has a radius of 20
  self.shape = love.physics.newCircleShape(20-0.65)
  -- Attach fixture to body and give it a density of 1.
  self.fixture = love.physics.newFixture(self.body, self.shape, 1)
  self.fixture:setRestitution(0.3) -- let the player bounce
  self.fixture:setCategory(2)
  self.body:setFixedRotation(true)

  self.flightMode = "walking"
  self.tucked = false
  self.flapCooldown = 8/12
  self.timeOfLastFlap = 0
  self.flapping = false
  self.dragVector = Vec2(0,0)
  self.liftVector = Vec2(0,0)
  self.normalFiction = 2
  self.fixture:setFriction(self.normalFiction)
  self.correctionRotationVelocity = 0

  self.fixture:setUserData(self)
  self.contactWithGround = false
  self.tangentWithGround = Vec2(-1, 0)
  self.normalWithGround = false
  self.velocityLastFrame = Vec2(0,0)

  

  self.tangentVelocity = Vec2(0,0)
  velocityVector = Vec2(0,0)

  self.collisionTable = {}
  self.collisionVectorTable = {}
  self.bonked = 0
  self.bonkImmunity = 0
  
  
  
  self.stasis = false
  self.stasisCountdown = 0
  self:resetParryWindow()

  
  self.wetness = 0
  --self.numberOfSwords = 0

  self.textureSet = "drea"
  self.lookDirection = 1
  self.walkRotation = 0
  self.animation = nil

  self.animationInfo = {}
  self.animationInfo.walking = nil
  self.animationInfo.lastHitGround = nil
  self.animationInfo.timeOfLastContact = 0

  self.anim = {}
  self.anim.running  = AnimationPlayer(animation.drea_running)
  self.anim.flapping = AnimationPlayer(animation.drea_flapping)
  self.anim.gliding  = AnimationPlayer(animation.drea_gliding)

  self.waterParry = 0
  self.parried = 0
  self.magicParticleGen = ParticleGen(0,0, 10.0, texture.magic_decay)

  self.fire = true
  self.timeBubble = false

  self.timeWarp = 2
  
  self.trail = TrailRibbon()
  self.trailOpacity = 0
  
  
  self.sleepTimer = 0
  
end


for i, file in ipairs(love.filesystem.getDirectoryItems("entities//player")) do
  local address = "entities.player."..file:sub(1, -5)
  --I am genius
  if "entities.player.player" ~= address then
    require(address)
  end
end 



function Player:setPosition(x, y)

  if not y then
    self.body:setPosition(x.x, x.y)
  else
    self.body:setPosition(x, y)
  end

end


function Player:getVelocity()
  return Vec2(self.body:getLinearVelocity())
end


function Player:getX()
  return self.body:getX()
end


function Player:getY()
  return self.body:getY()
end

local SLEEP_TIMER_LENGTH = 30
function Player:stopSleeping()
  self.sleepTimer = SLEEP_TIMER_LENGTH
end


function Player:snapCamera()
  camera:setPosition(self:getX(), self:getY())
end


function Player:spawn(spawnPointName)
  
  local spawnPoints = nameIndex.entities[spawnPointName]
  if spawnPoints then
    --If there's multiple, pick a random one
    local spawnPoint  = spawnPoints[love.math.random(1, #spawnPoints)]
    self:setPosition(spawnPoint:getPosition())
    self:snapCamera()
  else
    cout("Warning - no spawn point named '".. spawnPointName.."'")
  end

end
  

function Player:redirect(x, y)
  
  local speed = vector.getMagnitude(Vec2(self.body:getLinearVelocity()))

  if not y then
    --If the first argument is a vector, use it
    
    
    self.body:setLinearVelocity(x.x*speed, x.y*speed)
    self.directionVector = x
  else
    
    self.body:setLinearVelocity(x*speed, y*speed)
    self.directionVector = Vec2(x, y)
  end

end






function Player:timeBubbleBehaviour()

  if controls.timeBubble.thisFrame then
    
    
    
    if self.timeBubble then
      sound.effect.stasisOff:clone():play()
      self.timeBubble = false
    else
      sound.effect.stasis:clone():play()
      self.timeBubble = 150
      shader.timeBubble:send("bubbleRadius", self.timeBubble)
    end
  end
  
end


function Player:isParrying()
  return self.parried > 0 and controls.tuck.isDown
end


function Player:withinParryWindow()
  local t = 1 - (self.timeTucked)/self:getParryWindow()
  return 0 < t, 
  math.max(t, 0)
end


function Player:resetParryWindow()
  self.timeTucked = 1/0
end



--[[
function Player:parryIndicatorUpdate(dt)
  
  if love.keyboard.isDown("t") then
    
    self:showParryIndicator()
    
  end
  
  local timerMax = 0.2
  
  if self.indicatorLightTimer then
    if self.indicatorLightTimer > timerMax then
      self.indicatorLightTimer = nil
    else
      self.indicatorLightTimer = self.indicatorLightTimer + dt
    end
  end
  
end]]

--[[
function Player:predictParry()
  
  local parryWindow = self:getParryWindow()
  
  
  --Since the vector is in units per second, we can get the distance without normalizing
  self.parryWindowDistance = self:getVelocity() * parryWindow
  
  
  
  
  self.parryWindowDistance - vector.normalize(self:getVelocity()) * self.shape:getRadius()*1.5
  
  world:rayCast( x1, y1, x2, y2, function() self:showParryIndicator() end )

  
end]]




function Player:getParryWindow()
  return 0.5
end


function Player:isAboveParrySpeed()
  --Needs to use velocity last frame as it reduces once you hit something
  
  return vector.getMagnitudeSqrd(self.velocityLastFrame) > 1000^2
  
end


local parryTime = 2
function Player:stickyParryBehaviour(dt)
  
  self.timeTucked = self.timeTucked + dt
  
  if controls.tuck.isDown then
    self:getStickyVector()
  end
  

  self.parried = self.parried - dt
  self.waterParry = self.waterParry - dt

  local _, normal = next(self.collisionTable)
  
  local parried           = self.parried < 0
  local hasContact        = normal ~= nil
  local waterParryWindow  = self.waterParry > 0
  local notBonked         = self.bonked <= 0
  
  
  --Changing it so that it doesn't need a change in velocity, but a high enough velocity. TODO change the naming if this ends up sticking

  if self.tucked and hasContact and notBonked and ((parried and self:withinParryWindow() and self:isAboveParrySpeed()) or waterParryWindow) then
    self:onParry()
  elseif self.parried > 0 then
    
    --Detaching code 
    
    local vel = Vec2(self.body:getLinearVelocity())
    local pos = Vec2(self.body:getPosition())
    
    local angleGreaterThan90 = self.stickAngleWithLast and math.deg(self.stickAngleWithLast) >= 90
    --Detach if distance from the stick point is more than 3 times the step
    
    local speedDistance = vector.getMagnitude(self.stickPoint - pos) > vector.getMagnitude(vel)*3*dt
    
    
    if self.stickPoint and (angleGreaterThan90 or speedDistance) then
      self.parried = 0
    end
  end


  if self.parried > 0 and controls.tuck.isDown then


    local velocityVector
    local speed 
    if parryTime == self.parried then
      --If this is the frame that it was parried, use the velocity last frame to prevent slowing down due to bouncing
      velocityVector = self.velocityLastFrame
      speed = vector.getMagnitude(velocityVector)*0.8
      
      
      local angularVelocity = speed/self.shape:getRadius()
      
      if vector.angleIsClockwise(normal, velocityVector) then
        self.body:setAngularVelocity(angularVelocity)
      else
        self.body:setAngularVelocity(-angularVelocity)
      end
      
    else
      velocityVector = self:getVelocity()
      speed = vector.getMagnitude(velocityVector)
    end


    local newVelocityVector = vector.normalize(vector.project(velocityVector, self.stickyDirection))* speed

    self.body:setLinearVelocity(newVelocityVector.x, newVelocityVector.y)

    self.body:setPosition(self.stickPoint.x, self.stickPoint.y )
    
    

  end
  
end


function Player:getHeadPosition()
  local offset = self.directionVector * self.shape:getRadius() +  Vec2(0, -5)
  return self:getPosition() + offset
end


function Player:onParry()
  
  local sound = sound.effect.bounce_parry:clone()
  sound:play()
  self.parried = parryTime
  self.waterParry = 0
  
  local intensity = 4
  local lightTime = 0.2
  LingerLight(self:getX(), self:getY(), -5, intensity, {189/255, 249/255, 1}, intensity/lightTime)
  
end


function Player:fireballBehaviour()
  
  if self.fire and controls.fireball.thisFrame and (not self.tucked) and (not self.animationInfo.isSqueezing) and (not self.inWater) then
    local magnitude = 700

    local vx, vy = self.body:getLinearVelocity()

    
    local launchPosition = self:getHeadPosition()
    local direction
    if gamepadMode then
      local target = vector.normalize(Vec2(joysticks[1]:getGamepadAxis("rightx"), joysticks[1]:getGamepadAxis("righty")))
      direction = vector.normalize(target)
    else
      local target = Vec2(mouse.worldX, mouse.worldY)
      direction =  vector.normalize(target - launchPosition)
    end
    

    
    --Direction vector does not work properly as an offset
    Fireball(
      launchPosition.x, 
      launchPosition.y, 
      direction.x*magnitude + vx,
      direction.y*magnitude + vy)
    self.fire = false
    self:stopSleeping()
  end
end



function Player:stopSqueezing()
  self.sqeezeTrack = nil
  self.body:setActive(true)
  self.animationInfo.isSqueezing = nil
  self.parried = 0
end


function Player:squeezeBehaviour(dt)
  
  self.wetness = self.wetness - dt
  
  local squeezeTrack = self.sqeezeTrack
  
  if squeezeTrack then
    
    local newPos, dir = squeezeTrack:getPositionFromDistance(self.squeezeDistance + self.squeezeSpeed*dt)
    
    --Accerelerate due to gravity
    
    local _, gravity = world:getGravity()
    
    
    local accelleration = gravity * self.directionVector.y * 0.8
    
    self.squeezeSpeed = self.squeezeSpeed + accelleration*dt
    
    
    --Main movement
    self.squeezeDistance = self.squeezeDistance + self.squeezeSpeed*dt
    
    if newPos then
      if self.squeezeSpeed < 0 then
        dir = dir *-1
      end
        
      self.directionVector = dir
      local vel = dir * math.abs(self.squeezeSpeed)
      self.body:setActive(false)
      self.body:setPosition(newPos.x, newPos.y)
      self.body:setLinearVelocity(vel.x, vel.y)
      
    else
      self:stopSqueezing()
    end
    
    
    
    
  end
  
end
  


function Player:isStuck()
  
  local normals = self.collisionTable
  
  return normals[2] and vector.angleBetween(normals[1], normals[2]) >= math.rad(90)

  
end



function Player:interactBehaviour()

  if self.tucked == false and self.flightMode == "walking" then
    local closestInteractable = self:checkInteractables()
    if closestInteractable then
      closestInteractable.nearest = true
    end

    if controls.interact.thisFrame then
      self:interact(closestInteractable)
    end
  end
end




function Player:getStickyVector()
  --Now need to find the closest point

  local closestDistanceSqrd
  local closestPoint
  local stickyDirection
  self.stickPoint = nil

  for i, polygon in ipairs(polygonTable) do

    if polygon.stickTracks then
      for i = 1, #polygon.stickTracks, 2 do

        local x1 = polygon.stickTracks[i]
        local y1 = polygon.stickTracks[i+1]

        local x2 = polygon.stickTracks[(i+1) % #polygon.stickTracks + 1]
        local y2 = polygon.stickTracks[(i+2) % #polygon.stickTracks + 1]


        local point = line.getClosestPoint(x1, y1, x2, y2, self.body:getPosition())
        local displacementVector = vector.subtraction(point, Vec2(self.body:getPosition()))
        local distanceSqrd = vector.getMagnitudeSqrd(displacementVector)

        if (not closestDistanceSqrd) or distanceSqrd < closestDistanceSqrd then
          closestPoint = point
          closestDistanceSqrd = distanceSqrd

          stickyDirection = Vec2(x2 - x1, y2 - y1)
          
          if distanceSqrd < 0.1^2 then
            break
          end
          
        end
      end
    end
  end
  if self.stickyDirection and stickyDirection then
    self.stickAngleWithLast = vector.angleBetween(self.stickyDirection, stickyDirection)
  end
  self.stickyDirection = stickyDirection
  self.stickPoint = closestPoint

end


function Player:getBehindPolygon()
  
  for i, polygon in ipairs(polygonTable) do
    if not polygon.attributes.collidable then
      if polygon:checkInside(self.body:getPosition()) then
        return polygon
      end
    end
  end
  return false
  
end




local function getDistanceVector(object, playerPos)
  
  if object.type == "squeeze" then
    
    local point1 = Vec2(object.curve:getControlPoint(1))
    local pointN = Vec2(object.curve:getControlPoint(-1))
    local displacement1 = point1 - playerPos
    local displacement2 = pointN - playerPos
    local distance1 = vector.getMagnitudeSqrd(displacement1) 
    local distance2 = vector.getMagnitudeSqrd(displacement2) 
    
    if distance1 < distance2 then
      object.x = point1.x
      object.y = point1.y
      return displacement1, distance1
    else
      object.x = pointN.x
      object.y = pointN.y
      return displacement2, distance2
    end
    
  else
    return object - playerPos, vector.getMagnitudeSqrd(object - playerPos)
  end
  
end

local interactableTable = require "tables.interactables"

function Player:checkInteractables()
  --Finally, inner peace
  local playerPos = Vec2(self.body:getPosition())
  local closestDistance = nil
  local closestObject = nil
  local priority = false
  
  for i, object in ipairs(interactableTable) do
    --Note to self - might be better to only pass a table of interactables
    --Further note to self - this might also be an opitunity to implement quadtrees
    if object:isInteractable() then
      --Priority objects will always get through
      if (not priority) or object.attributes.priority then
        --Check if the player is facing the object if it needs to
        
        local distanceVector, newDistance = getDistanceVector(object, playerPos)
        if object.attributes.requiresFacing == false or self.directionVector.x * (object.x - playerPos.x) > 0 then

          
          --Check if the object is within the interact radius
          if newDistance < object.attributes.interactRadius^2 then
            
            --Set default values if no object has been found yet
            closestDistance = closestDistance or newDistance
            closestObject   = closestObject or object
            
            --Check if closer than the closest so far. If not, check if it is a priority object, and if priority mode has not already been set, select this as the closest object
            if newDistance < closestDistance or (object.attributes.priority and not priority) then
              priority        = priority or object.attributes.priority
              closestDistance = newDistance
              closestObject   = object
            end
          end
        end
      end
    end
  end
  
  return closestObject
end


function Player:interact(closestObject)
  if closestObject then
    local relativePlayerPosition = self:getPosition() - closestObject
    closestObject:interact(relativePlayerPosition)
  end
end


function Player:keepInBounds()
  local x = self:getX() 
  local y = self:getY() 

  if x > 60000 then
    self.body:setX(-30000)
    camera.position.x = camera.position.x -30000
  end

  if x < -60000 then
    self.body:setX(30000)
    camera.position.x = camera.position.x + 30000
  end



  if y > 15000 then
    local diffX = self:getX()
    local diffY = self:getX() - -10000
    
    self.body:setPosition(0, -10000)
        
    camera.position.x = camera.position.x + diffX
    camera.position.y = camera.position.y + diffY
  end

end



function Player:getMouseRelativeToPlayer()
  return Vec2(mouse.worldX, mouse.worldY) - self:getPosition()
end


function Player:getTailLength()
  if self:isBoundryBreaking() then
    return math.max(vector.getMagnitude(self:getVelocity())*0.6, 100)
  else
    return math.max(vector.getMagnitude(self:getVelocity())*0.2, 100)
  end
end


function Player:updateTrail(dt)
  local pointDensity = 10
  local pos = self:getPosition()
  local x, y = pos.x, pos.y
  
  self.trail:setPosition(x, y)
  

  local length = self:getTailLength()
  
  
  local pointDistance = length/pointDensity
  
  local pointDistanceSqrd = self.trail:distanceToFirstPointSqrd()

  if self:isAboveParrySpeed() then
    self.trailOpacity = math.min(self.trailOpacity + dt, 1)
  else
    self.trailOpacity = math.max(self.trailOpacity - dt, 0)
  end
  
  
  
  if (not pointDistanceSqrd or (pointDistanceSqrd > pointDistance^2 and self.trailOpacity > 0)) then
    self.trail:addPoint(x, y)
  end
  
  if self.trailOpacity == 0 then
    self.trail:resetPoints()
  end
  
  
  self.trail:trimPoints(length)
end



function Player:update(dt)
  
  self.fire = true
  velocityVector = self:getVelocity()


  self:getCollisionVectors()

  if self.character == "Drea" or true then
    
    

    
    
--    self:modeSwitchBehaviour(dt)
    self:squeezeBehaviour(dt)
    self:tuckingBehaviour(dt)
    self:flyingBehaviour(dt)
    self:blowbackBehaviour(dt)
    --self:stasisBehaviour(dt)
    --self:boostBehaviour(dt)
    --self:bounceParryBehaviour()
    self:stickyParryBehaviour(dt)
    self:timeBubbleBehaviour()
    self:fireballBehaviour()
    self:waterBehaviour(dt)
    self:updateTrail(dt)
    
    self.sleepTimer = self.sleepTimer - dt
    self.animationInfo.sleeping = self.sleepTimer <= 0

    if self:isStuck() then
      self.fixture:setFriction(0)
    else
      self.fixture:setFriction(self.normalFiction)
    end
    

    self:keepInBounds()

    self.falling = (not self.tucked) and self.flightMode == "walking" and love.timer.getTime() - self.animationInfo.timeOfLastContact > 0.3

    if self.falling then

      local vx, vy = self.body:getLinearVelocity()
      
      if not (vx == 0 and vy == 0) and not self.animationInfo.isSqueezing then
        self.directionVector = vector.normalize(Vec2(vx, vy))
      end
    end
  end



  if self.tucked == false and self.flightMode == "walking" then
    self:walkingBehaviour()
  end



  

  if self.character == "Sift" then
    --[[if clickedThisFrame.primary and self.numberOfSwords < 2 then
      --Note - fix crash when mouse is at 0,0
      self.numberOfSwords = self.numberOfSwords + 1
      local distanceFromPlayer = 50
      local swordPosition = vector.normalize(mouseRelativeToPlayer) * (20 + distanceFromPlayer)
      local newSword = Sword(swordPosition.x + self.body:getX(), swordPosition.y + self.body:getY(), mouseRelativeToPlayer)
      
      
      if self.numberOfSwords == 1 then
        self.rope = Rope(self, newSword, 1500)
      elseif self.numberOfSwords == 2 then
        newSword.rope = self.rope
        self.rope:setLength(self.rope.defaultLength)
        self.rope:swapBodies(self.body, newSword.body)
      end
      
    end
    if love.mouse.isDown(2) then
      self.numberOfSwords = 0
    end
    
    self:airWalkingBehaviour()
    ]]
  end 

  if controls.lockDirection.thisFrame then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.isDown then
    mouse.storedX = love.mouse.getX()
    mouse.storedY = love.mouse.getY()
  end
  if controls.lockDirection.upThisFrame then
    love.mouse.setPosition(mouse.storedX, mouse.storedY)
  end


  self:interactBehaviour()


  if self.contactWithGround then
    self.animationInfo.timeOfLastContact = love.timer.getTime()
  end


  self.magicParticleGen.rotation = vector.getOrientation(self:getVelocity()) + math.rad(90)
  self.magicParticleGen.x = self.body:getX()
  self.magicParticleGen.y = self.body:getY()
  local minMagic = 0.5
  local maxMagic = 3
  self.magicParticleGen.amount = math.floor(vector.getMagnitudeSqrd(self:getVelocity())/(160)^2) * math.min(math.max(self.parried*2, 0), maxMagic)/3

  

  for i, animation in pairs(self.anim) do
    animation:update(dt)
  end


  self.velocityLastFrame = self:getVelocity()


end



function Player:drawTrail()
  blue.graphics.setZ(0)
  if self.trailOpacity > 0 then
    
    if not self:isBoundryBreaking() then
      do
        local _, t = self:withinParryWindow()
        if self:isParrying() then
          t = math.max(self.parried/parryTime, 0)
        elseif self.bonked > 0 then
          t = 0
        end
        
        t = t^0.5
        
        local r = lume.lerp(1, PARRY_COLOUR[1], t) 
        local g = lume.lerp(1, PARRY_COLOUR[2], t) 
        local b = lume.lerp(1, PARRY_COLOUR[3], t) 
        local a = lume.lerp(0.08*self.trailOpacity, 0.5*self.trailOpacity, t)
        love.graphics.setColor(r,g,b,a)
      end
    else
      local a = 0.5*self.trailOpacity
      love.graphics.setColor(107/255, 250/255, 255/255, a)
    end
    
    local trailPoints = self.trail:getPoints()
    self.trail:drawTriangle(trailPoints, 7, self:getTailLength())
    
    
    if debugMode then
      for i = 1, #trailPoints do
        local x = trailPoints[i].x
        local y = trailPoints[i].y
        
        if i == #trailPoints then
          love.graphics.setColor(0,1,0)
        else
          love.graphics.setColor(1,0,0)
        end
        
        blue.graphics.print(i, x, y)
        blue.graphics.circle("line", x, y, 2)
      end
    end
    
  end
end


function Player:draw()
  
  blue.graphics.setZ(0)
  
  
  --Beginning of texture drawing
  local oldShader = love.graphics.getShader()
  love.graphics.setShader(shader.energy)
  
  
  

  local rot = vector.getOrientation(self.directionVector) + math.rad(90)

  if not (self.tangentWithGround.x == 0 and self.tangentWithGround.y == 0) then
    self.walkRotation = vector.getOrientation(self.tangentWithGround) + math.rad(-90)
  else
    if (self.directionVector.x) < 0 then
      self.walkRotation = rot + math.rad(180)
    else
      self.walkRotation = rot
    end
  end
  

  
  local dryness = 1 - math.min(self.wetness*0.25, 0.5)
  love.graphics.setColor(dryness, dryness, 1)
  
  
  if (self.flightMode ~= "walking" or self.falling) and ((not self.tucked) or self.animationInfo.isSqueezing) then
    
    self.lookDirection = lume.sign(self.directionVector.x)

  end

  -- Only one animation at a time! self.animation refers to full animations here, or static images

  if self.animationInfo.sleeping then
    self.animation = "sleeping"
  elseif self.animationInfo.isSqueezing then
    self.animation = "squeezing"
  elseif self.animationInfo.walking then
    self.animation = "running"
  elseif self.tucked then
    self.animation = "tucked"
  elseif self.falling then
    self.animation = "falling"
  elseif self.flightMode == "swooping" and not self.tucked and (self.flapping or self.isFlapCooldown) then
    self.animation = "flapping"
  elseif self.flightMode == "swooping" then
    self.animation = "gliding"
  elseif self.animationInfo.sliding then
    self.animation = "sliding"
  else
    -- If nothing else is detected, then set to idle
    self.animation = "idle"
  end


  do 
    local sts = smallTextureScale
    local x, y = self.body:getPosition()

    if self.animation == "idle" then
      local texture = texture.drea_idle
      blue.graphics.drawCentredShaded(texture, x, y, self.walkRotation, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "running" then
      self.anim.running:play(x, y, self.walkRotation, sts*self.lookDirection, sts)
    elseif self.animation == "tucked" then
      local texture = texture.drea_tucked
      blue.graphics.drawCentredShaded(texture, x, y, rot, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "sliding" then
      local texture = texture.drea_sliding
      blue.graphics.drawCentredShaded(texture, x, y, self.walkRotation, sts*self.lookDirection, sts, texture.ox, texture.oy)
    elseif self.animation == "falling" then
      local texture = texture.drea_falling
      blue.graphics.drawCentredShaded(texture, x, y, rot, sts, sts*self.lookDirection, texture.ox, texture.oy)
    elseif self.animation == "gliding" or self.animation == "squeezing" then
      self.anim.gliding:play(x, y, rot, sts, sts*self.lookDirection)
    elseif self.animation == "flapping" then
      self.anim.flapping:play(x, y, rot, sts, sts*self.lookDirection)
    elseif self.animation == "sleeping" then
      local texture = texture.drea_sleeping
      blue.graphics.drawCentredShaded(texture, x, y, 0, sts, sts, texture.ox, texture.oy)
    end
  end
  

  for i, animation in pairs(self.anim) do
    if i ~= self.animation then
      animation:stop()
    end
  end
  
  if self.parried > 0 then
    local x = (parryTime - self.parried)/parryTime
    local intensity = (-x*x + 1) * 2
    addLight(self:getX(), self:getY(), -40, intensity, PARRY_COLOUR)
  end
  
  --Add parry indicator light
  
  
  if self.indicatorLightTimer then
    addLight(self:getX(), self:getY(), -40, self.indicatorLightTimer, PARRY_COLOUR)
  end
  
  love.graphics.setShader(oldShader)
  
end


function Player:drawOverlay()

  
  if debugMode then 
    if self:isStuck() then
      love.graphics.setColor(0,0,1, 0.5)
    else
      love.graphics.setColor(1,0,0, 0.5)
    end
    blue.graphics.circle("fill", self.body:getX(), self.body:getY(), self.shape:getRadius())
    love.graphics.setColor(0, 0, 0)
    blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius())


    if self.flightMode == "swooping" and self.tucked == false then
      love.graphics.setColor(0, 0.5, 1)
      blue.graphics.line(self.body:getX() + self.directionVector.x*-80, self.body:getY() + self.directionVector.y*-80, self.body:getX() + self.directionVector.x*80, self.body:getY() + self.directionVector.y*80)
    end


    if self.tucked then
      love.graphics.setColor(0,0,0,0.5)
      blue.graphics.circle("line", self.body:getX(), self.body:getY(), self.shape:getRadius()+5)
    end


    love.graphics.setColor(0, 0, 0)
    blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.directionVector, 35)
    
    
    
    for i, normal in pairs(self.collisionTable) do
      love.graphics.setColor(0, 0.9, 0.5)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), normal, 60)
    end


    if self.normalWithGround and self.contactWithGround then
      love.graphics.setColor(1, 0.9, 0.5)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.normalWithGround, 60)
    end

    if self.stickyVector then
      love.graphics.setColor(1, 0, 0, 0.2)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), self.stickyVector, 60)
    end
    
    if self.blowbackVector and controls.blowbackMode.isDown then
      love.graphics.setColor(1, 1, 1)
      blue.graphics.vectorLine(self.body:getX(), self.body:getY(), vector.normalize(self.blowbackVector), 35)
    end
    
    if self.closestPoint then

      love.graphics.setColor(0, 1, 1)
      blue.graphics.circle("line", self.closestPoint.x, self.closestPoint.y, 5)
      
      love.graphics.setColor(0, 0, 1)
      blue.graphics.vectorLine(self.closestPoint.x, self.closestPoint.y, self.closestPointNormal, 50)
      love.graphics.setColor(1, 0, 0)
      blue.graphics.vectorLine(self.closestPoint.x, self.closestPoint.y, self.closestPointNormal, -50)
      
    end
    
    do
    
      local contactNormal = Vec2(0,0)
      
      for i, normal in pairs(self.collisionTable) do
        contactNormal = contactNormal + normal
      end
      contactNormal = vector.normalize(contactNormal)
      
      if contactNormal.x == 0 and contactNormal.y == 0 then
        contactNormal = self.lastContactNormal
      else
        self.lastContactNormal = contactNormal
      end
      
      --[[if contactNormal then
        local direction
        if vector.dotProduct(contactNormal, mouseRelativeToPlayer) > 0 then
          direction = mouseRelativeToPlayer
        else
          local mirrorLine = vector.turn90Degrees(contactNormal)
          direction = (mouseRelativeToPlayer - 2 * vector.dotProduct(mouseRelativeToPlayer, mirrorLine) * mirrorLine)*-1
          
        end
        love.graphics.setColor(1,0,0)
        blue.graphics.vectorLine(self.body:getX(), self.body:getY(), direction)
      end]]
    end
    
  end
  
end



function Player:pixelCanvasDraw()
  
  
  lg.push()
  camera:applyTransformations()
  
  local oldShader = love.graphics.getShader()
  love.graphics.setShader(shader.water)
  
  local width = self.attributes.width * smallTextureScale
  local height = self.attributes.height * smallTextureScale
  
  
  
  local poly = self:getPolygon()
  
  
  lg.setColor(1,1,1)
  blue.graphics.circle("fill", self.body:getX(), self.body:getY(), 10)
  
  love.graphics.setShader(oldShader)
  
  lg.pop()
end