local waterSources = require "tables.waterSources"
function Player:waterBehaviour(dt)
  local x, y = self.body:getPosition()
  
  
  for i, source in ipairs(waterSources) do
    --cout(i)
    if source:checkInside(x, y) then
      self:inWaterBehaviour(dt, source)
      self.inWater = true
      self.timeInWater = (self.timeInWater or 0) + dt
      return
    end
  end
  
  if self.inWater then
    self:leaveWater()
    self.inWater = false
  end

  
end


function Player:skip()
  
  if self.tucked then
    local v = self:getVelocity()
    
    --The horizontal direction doesn't matter.

    if v.y > 0 and math.abs(v.x/love.physics.getMeter()) > 15 and math.rad(40) >= math.atan(v.y/math.abs(v.x)) then
      self.body:setLinearVelocity(v.x*0.8, -v.y*0.65)
      self.waterParry = 3
      return true
    else
      self.waterParry = 0
    end
  end
  return false
end 



--[[
function Player:inWaterBehaviour(dt, waterSource)
  
  
  if not self.inWater then
    self:enterWater(waterSource)
  end
  
  
  if (self.waterParry <= 0 or not self.tucked) and not (self.tucked and self.parried > 0) then
    self.wetness = math.max(self.wetness, waterSource.attributes.wetness)
  end
  
  self.flapping = false
  self:smoothRotation(dt, math.max(300 / (vector.getMagnitude(self:getVelocity())*0.002+1), 50))
  
  
  
  local minDragSpeed
  local linearDrag = 2
  
  if controls.fireball.thisFrame and not self.tucked then
    
    local impluse = 100
    self.body:applyLinearImpulse(self.directionVector.x * impluse, self.directionVector.y * impluse)
    
  end
  
  
  if controls.flap.isDown and not self.tucked then

    local force = 100
    self.body:applyForce(self.directionVector.x * force, self.directionVector.y * force)
    minDragSpeed = 3
    
    self.animationInfo.sleeping = false
    self.animationInfo.walking = false
    self.flightMode = "swooping"
    
  elseif self.tucked == false and self:canWalk() then
    self.flightMode = "walking"
  else
    minDragSpeed = 0
  end
  
  
  
  if self.tucked == false then
    self:applyAerodynamics(dt, {minDragSpeed = minDragSpeed, linearDrag = linearDrag, wingDrag = 3})
  elseif not self:isParrying() then
    --Apply linear drag
    local vx, vy = self.body:getLinearVelocity()
    self.body:applyForce(
      -vx * 1.5, 
      -vy * 1.5)
    
    
    --Apply angular drag
    local rotSpeed = self.body:getAngularVelocity()
    self.body:applyTorque(-rotSpeed*100)

  end
  
  
  if self.inWater then
    
    local trailTime = 0.25

    
    local bubbleSpawn = self.bubbles/(trailTime - self.bubbleTimer)*dt
    
    
    self.bubbles = math.max(self.bubbles - bubbleSpawn, 0)
    self.bubbleTimer = self.bubbleTimer + dt
    
    while bubbleSpawn > 0 do
      
      local ox, oy = randomPointInCircle(self.shape:getRadius()/2)
      local vx, vy = self.body:getLinearVelocity()
      local vox, voy = randomPointInCircle(100)
      
      vx = vx + vox
      vy = vy + voy
      
      Bubble(self.body:getX()+ox, self.body:getY()+oy, vx, vy, waterSource.y - waterSource.height*0.5)
      
      bubbleSpawn = bubbleSpawn - 1
    end

  end
  
  local _, gravity = world:getGravity()
  local bouyantForce = self.body:getMass()*-gravity*0.85
  self.body:applyForce(0, bouyantForce)
  
  
  --self:applyWingFlapForce(dt)
end]]


function Player:inWaterBehaviour(dt, waterSource)
  
  
  if not self.inWater then
    self:enterWater(waterSource)
  end
  
  local minDragSpeed
  if controls.flap.isDown and not self.tucked then

    local force = 100
    self.body:applyForce(self.directionVector.x * force, self.directionVector.y * force)
    minDragSpeed = 8
    
    self:stopSleeping()
    self.animationInfo.walking = false
    self.flightMode = "swooping"
    
  elseif self.tucked == false and self:canWalk() then
    self.flightMode = "walking"
  else
    minDragSpeed = 0
  end
  
  
  
  if (self.waterParry <= 0 or not self.tucked) and not (self.tucked and self.parried > 0) then
    self.wetness = math.max(self.wetness, waterSource.attributes.wetness)
  end
  
  self.flapping = false
  self:smoothRotation(dt, math.max(300 / (vector.getMagnitude(self:getVelocity())*0.002+1), 50))
  
  
  
  if self.tucked == false then
    self:applyAerodynamics(dt, {minDragSpeed = minDragSpeed, linearDrag = 2, wingDrag = 3})
  elseif not self:isParrying() then
    --Apply linear drag
    local vx, vy = self.body:getLinearVelocity()
    self.body:applyForce(
      -vx * 1.5, 
      -vy * 1.5)
    
    
    --Apply angular drag
    local rotSpeed = self.body:getAngularVelocity()
    self.body:applyTorque(-rotSpeed*100)

  end
  
  
  if self.inWater then
    
    local trailTime = 0.25

    
    local bubbleSpawn = self.bubbles/(trailTime - self.bubbleTimer)*dt
    
    
    self.bubbles = math.max(self.bubbles - bubbleSpawn, 0)
    self.bubbleTimer = self.bubbleTimer + dt
    
    while bubbleSpawn > 0 do
      
      local ox, oy = randomPointInCircle(self.shape:getRadius()/2)
      local vx, vy = self.body:getLinearVelocity()
      local vox, voy = randomPointInCircle(100)
      
      vx = vx + vox
      vy = vy + voy
      
      Bubble(self.body:getX()+ox, self.body:getY()+oy, vx, vy, waterSource.y - waterSource.height*0.5 + waterSource:getLevelOffset())
      
      bubbleSpawn = bubbleSpawn - 1
    end

  end
  
  local _, gravity = world:getGravity()
  local bouyantForce = self.body:getMass()*-gravity*0.85
  self.body:applyForce(0, bouyantForce)
  
  
  --self:applyWingFlapForce(dt)
end


function Player:enterWater(source)
  
  self.inWater = true
  self.timeInWater = 0
  sound.effect.splash:clone():play()
  
  
  self.bubbles = 400 * vector.getMagnitudeSqrd(self:getVelocity()/love.physics.getMeter()) / (30*30)
  self.bubbleTimer = 0
  
  if not self:skip() then
    
    
  end
  
  
end


function Player:leaveWater()
  
  --Give a boost when leaving water
  if controls.flap.isDown and not self.tucked then

    local force = vector.getMagnitude(self:getVelocity())*10/love.physics.getMeter()

    self.body:applyLinearImpulse(self.directionVector.x * force, self.directionVector.y * force)
  end
  
end



