TrailRibbon = Entity:extend()
local lg = love.graphics

--How this works - points are added as it moves, and the last points are removed as they go outside of the range. The very last point will be out of range, so that the tip of the trail can interpolate between them

function TrailRibbon:new(x, y)
  
  self.points = {}
  
end


function TrailRibbon:addPoint(x, y)
  local point = Vec2(x, y)
  --As the position of the points don't change, I will store the length to the next one here
  local nextPoint = self.points[1]
  
  if nextPoint then
    point.length = vector.getMagnitude(point - nextPoint)
  end
  
  table.insert(self.points, 1, point)
end


function TrailRibbon:distanceToFirstPointSqrd()
  if self.points[1] then
    return vector.getMagnitudeSqrd(Vec2(self.x, self.y) - self.points[1])
  end
  
  return false
end


function TrailRibbon:trimPoints(length)
  if self.points[1] then
    local cumulativeLength = math.sqrt(self:distanceToFirstPointSqrd())
    local cutoffIndex
    local endPointLength
    for i, point in ipairs(self.points) do
      
      
      
      if cumulativeLength >= length then
        cutoffIndex = i + 1
        
        if self.points[i-1] then
          endPointLength = length - (cumulativeLength - (self.points[i-1].length or 0))
       
          
          --cout(endPointLength)
          
          
          local direction = vector.normalize(point - (self.points[i-1] or point))
          
          
          self.endPoint = self.points[i-1] + direction*endPointLength
          
        end
        break
      end
      
      cumulativeLength = cumulativeLength + (point.length or 0)
      
    end
    
    if cutoffIndex then
      while #self.points > cutoffIndex do
        table.remove(self.points)
      end
    end

  end
end


function TrailRibbon:getPoints()
  --Returns the final points used for rendering.
  
  local points = {Vec2(self.x, self.y)}
  
  
  for i, point in ipairs(self.points) do
    if i >= #self.points-1 and self.endPoint then
      break
    end
    
    if not (i == 1 and point == Vec2(self.x, self.y)) then
      table.insert(points, point)
    end
    
  end
  
  table.insert(points, self.endPoint)
  

  return points
end


function TrailRibbon:resetPoints()
  self.points = {}
  self.endPoint = nil
end



function TrailRibbon:drawTriangle(trailPoints, topRadius, length)
  local drawPoints = {}
  
  if self.points[1] then
 
    local function insertPoint(point, i)
      i = i or #drawPoints + 1
      table.insert(drawPoints, i, point.y)
      table.insert(drawPoints, i, point.x)
    end
    
    local distances = {vector.getMagnitude(Vec2(self.x, self.y) - self.points[1])}
    
    for i, point in ipairs(trailPoints) do
      table.insert(distances, (point.length or 0) + distances[#distances])
    end
    
    

    local nextPoint
    for i = #trailPoints, 1, -1 do

      local point = trailPoints[i]
      
      if i == #trailPoints then
        insertPoint(point)
      else
        
        local lastPoint = trailPoints[i + 1]
        local nextPoint = trailPoints[i - 1] or point
        
        local lastDir = vector.normalize(point - lastPoint)
        local nextDir = vector.normalize(nextPoint - point)
        
        local normal = vector.turn90Degrees(vector.normalize(lastDir + nextDir))
        
        
        local t = distances[#trailPoints - i]/length
        local thickness = topRadius*t +1
        
        
        insertPoint(point + normal*thickness, 1)
        insertPoint(point - normal*thickness)
        
        
      end
      
    end
    
    
    
    if #drawPoints*0.5 >= 3 then
      blue.graphics.polygon("fill", drawPoints)
      if debugMode then
        love.graphics.setColor(0,0,1)
        blue.graphics.polygon("line", drawPoints)
        
        
        for i = 1, #drawPoints, 2 do
          local x = drawPoints[i]
          local y = drawPoints[i+1]
          blue.graphics.circle("line", x, y, 2)
        end
      end
    end
    
    
    
  end
end