local pixelCanvas = {}
local lg = love.graphics



function pixelCanvas:getSnapOffsets()
  local screenPixelSize = camera:getFinalZoom()*smallTextureScale
  
  --[[
  
  local snapOffsetX = (camera.x*camera:getFinalZoom() + window.width*0.5)%zoom
  local snapOffsetY = (camera.y*camera:getFinalZoom() + window.height*0.5)%zoom]]
  
  
  local x, y = blue.standardTransform(0,0) 
  
  local snapOffsetX = x%(screenPixelSize)
  local snapOffsetY = y%(screenPixelSize)
  
  --cout(snapOffsetX, screenPixelSize)
  
  return snapOffsetX, snapOffsetY, screenPixelSize
end


local function canvas_draw(self)
  
  
  local snapOffsetX, snapOffsetY, zoom = pixelCanvas:getSnapOffsets()
  
  if zoom > 1 then
    
    lg.push()
    lg.translate(snapOffsetX, snapOffsetY)
    lg.scale(zoom, zoom)
    lg.translate(-1,-1)
    
  end
  lg.setColor(1,1,1)
  
  
  if self.customDraw then
    self:customDraw()
  else
    --love.graphics.setBlendMode("alpha", "premultiplied")

    blue.graphics.applyShadingVars()
    lg.draw(self.canvas, 0, 0, 0)
  end
  
  if zoom > 1 then
    lg.pop()
  end
  --lg.setBlendMode("alpha")

end


local function canvas_drawTo(self)
  love.graphics.setBlendMode("alpha", "premultiplied")
  local snapOffsetX, snapOffsetY, zoom = pixelCanvas:getSnapOffsets()
  
  lg.push()
  if zoom > 1 then
    
    lg.translate(1,1)
    lg.scale(1/zoom, 1/zoom)
    lg.translate(-snapOffsetX, -snapOffsetY)
  end
  
  lg.setCanvas(self.canvas)
  lg.clear(0,0,0,0)
  for i, object in ipairs(drawTable) do
    
    local objectType = object.sortType or object.type
  
    if objectType == self.type then
      if object.pixelCanvasDraw then
        object:pixelCanvasDraw()
      end
    end
    
    
  end

  lg.pop()
  
  love.graphics.setBlendMode("alpha")
end



function pixelCanvas.create(type, customDraw, settings)
  
  local newPixelCanvas = {}
  
  newPixelCanvas.draw   = canvas_draw
  newPixelCanvas.customDraw = customDraw
  newPixelCanvas.drawTo = canvas_drawTo
  newPixelCanvas.canvas = lg.newCanvas(love.graphics.getWidth()+1, love.graphics.getHeight()+1, settings)
   
  
  newPixelCanvas.type = type or "pixelCanvas"
  table.insert(drawTable, newPixelCanvas)
  
  return newPixelCanvas
end



return pixelCanvas