local handler = {lights = {}}
--This probably isn't the most efficient method as lights are constantly being added and deleted, but it's the best I could come up with


local function aabb_test(a,b)
  local x_condition = (a.left-b.right)*(b.left-a.right)
  local y_condition = (a.top-b.bottom)*(b.top-a.bottom)
  return math.min(x_condition, y_condition) > 0
end



local function rectIntersection(a, b)
  
  local ax1 = a.x - a.width*0.5
  local ax2 = a.x + a.width*0.5
  local ay1 = a.y - a.height*0.5
  local ay2 = a.y + a.height*0.5
  
  local bx1 = b.x - b.width*0.5
  local bx2 = b.x + b.width*0.5
  local by1 = b.y - b.height*0.5
  local by2 = b.y + b.height*0.5
  
  --https://discord.com/channels/329400828920070144/474705430434807819/1241069490045845545
  
  
  local x_condition = (ax1-bx2)*(bx1-ax2)
  local y_condition = (ay1-by2)*(by1-ay2)
  
  
  return math.min(x_condition, y_condition) > 0
  
end


local function cullLight(x, y, intensity)

  
  local radius = 100*intensity*camera.zoom

  
  x, y = blue.standardTransform(x, y)
  
  
  local lightSquare = {width = radius*2, height = radius*2, x = x, y = y}
  local cameraRect  = {width = window.width, height = window.height, x = window.width*0.5, y = window.height*0.5}
  
  return not rectIntersection(lightSquare, cameraRect)
  
end


function addLight(x, y, z, intensity, colour)
  if intensity ~= 0 and (not cullLight(x, y, intensity)) then
  
    local newLight = {}
    
    newLight.x = x
    newLight.y = y
    newLight.z = z
    
    
    if intensity == 0 then
      error("Cannot set light to an intensity of 0!", 2)
    end
    
    newLight.intensity = intensity
    newLight.colour = colour or {1,1,1}
    
    table.insert(handler.lights, newLight)
  end
end



local MAX_INTENSITY = 4
function handler:setAmbient(intensity, input_colour)
  
  --Since I don't want to use a different pixel format but want brightness higher than 1, I'll divide the intensity here and multiply it within the shader.
  
  
  
  
  input_colour = input_colour or {1,1,1}
  
  local colour = {}
  colour[1] = input_colour[1] * intensity/MAX_INTENSITY
  colour[2] = input_colour[2] * intensity/MAX_INTENSITY
  colour[3] = input_colour[3] * intensity/MAX_INTENSITY
  aColour = colour
end


function handler:getAmbient()
  return aColour
end




function handler:clearLights()
  handler.lights = {}
  
end

 

function handler:send()
  self:sendLights()
  self:clearLights()
end

local lastSentAmbient

local function sendToShader(shader, lightPositions, lightColours, lightIntensities)
  
  
  shader:send("lightIntensities", unpack(lightIntensities))
  
  
  if #lightPositions > 0 then
    shader:send("cameraZoom", camera.zoom)
    shader:send("lights", unpack(lightPositions))
    shader:send("lightColours", unpack(lightColours))
  end
  
  local ambientLight = handler:getAmbient()
  if (not lastSentAmbient) or (lastSentAmbient[1] == ambientLight[1] and lastSentAmbient[2] == ambientLight[2] and lastSentAmbient[3] == ambientLight[3]) then
    shader:send("outsideLight", ambientLight)
  end

end


function handler:sendLights()
  
  local maxLights = 32
  
  if settings.graphics.fastLighting then
    maxLights = 16
  end
  
  
  local lightPositions = {}
  local lightColours = {}
  local lightIntensities = {}
  
  for i = 1, maxLights do
    table.insert(lightIntensities, 0)
  end
  
  
  for i, light in ipairs(self.lights) do
    
    --Positions need to be traslated
    
    local x, y = blue.standardTransform(light.x, light.y)
    
    
    table.insert(lightPositions, {x, y, light.z})
    
    lightIntensities[i] = light.intensity
    table.insert(lightColours, light.colour)
    
    if i > maxLights then
      if not demoMode then
        cout("Exceeded max lights! ("..#self.lights..")")
      end
    end
    
  end
  
  --WARNING - a light of 0 intensity will disable all following lights 
  
  local activeShader
  if settings.graphics.fastLighting then
    activeShader = shader.lambertLightingSimple
  else
    activeShader = shader.lambertLighting
  end
  
  
  sendToShader(activeShader, lightPositions, lightColours, lightIntensities)

end





return handler