DialogueBox = require("lib.classic"):extend()
DialogueBox.type = "dialogueBox"
local lg = love.graphics

require "dialogue/characterInfo"
local dialogue = require "dialogue/dialogueManifest"
local bookFunctions = require "dialogue/bookFunctions"

function DialogueBox:new(x, y, bookName)
  
  self.x = x
  self.y = y

  self.fontScale = 2
  self.width = 1000
  self.height = font:getHeight()*self.fontScale*4 +20
  self.boarder = 10
  self.safeZoneX = 7
  
  self.pauseWorld = false
  
  if bookName then
    self:loadBook(bookName)
  end
  
  self.timeDisplayed = false
  self.printSpeed = 35
  table.insert(drawTable, self)
  
  self.bookQueue = {}
end


function DialogueBox:getDimensions()
  local scale = love.graphics.getWidth()/1920
  return self.width*scale, self.height*scale
end


function DialogueBox:updatePaused()
  if controls.interact.thisFrame and self.displayed then
    if #self.cutPage >= #self.currentPage then
      self:nextPage()
    elseif #self.cutPage > 0 then
      self.timeDisplayed = -1000
    end
  end
end



function DialogueBox:drawOverlay()
  
  --[[lg.push()
  
  local totalScale = lg.getWidth()/1920
  lg.translate(lg.getWidth()*0.5, lg.getHeight()*0.5)
  lg.scale(totalScale)
  lg.translate(-lg.getWidth()*0.5, -lg.getHeight()*0.5)]]
  
  
  if self.currentPage then
    local width, height = self.width, self.height
    
    love.graphics.setColor(1,1,1)
    love.graphics.rectangle("fill", 
      self.x - self.boarder - width*0.5 - self.safeZoneX, 
      self.y - self.boarder - height*0.5, 
      width + self.boarder*2 + self.safeZoneX*2, 
      height + self.boarder*2)
    
    love.graphics.setColor(0,0,0)
    love.graphics.rectangle("fill", 
      self.x - width*0.5 - self.safeZoneX, 
      self.y - height*0.5, 
      width + self.safeZoneX*2, 
      height)
    
    love.graphics.setColor(1,1,1)
    
    local _, lines = self.currentPage:gsub('\n', '\n')
    lines = lines + 1
    local currentChar = (love.timer.getTime( ) - self.timeDisplayed)*self.printSpeed
    
    
    self.cutPage = string.specialSubstring(self.currentPage, currentChar)
    
    if self.speakerName then
      --Draw speaker name tag
      local safeZomeX = 7
      local safeZomeY = 5
      local width = font:getWidth(self.speakerName)*self.fontScale + safeZomeX*2
      local height = font:getHeight()*self.fontScale + safeZomeY*2
      local boarderThickness = 10
      local x = self.x - self.width/2 - 50
      local y = self.y - self.height/2-height+1 -- boarderThickness
      
      
      
      
      love.graphics.setColor(1,1,1)
      love.graphics.rectangle("fill", x-boarderThickness, y-boarderThickness, width + boarderThickness*2, height + boarderThickness*2)
      love.graphics.setColor(0,0,0)
      love.graphics.rectangle("fill", x, y, width, height)
      
      local r, g, b = 1, 1, 1
      if characterInfo[self.speakerName] then
        r, g, b = characterInfo[self.speakerName]:getColour()
      end
      love.graphics.setColor(r, g, b)
      love.graphics.print(self.speakerName, x + safeZomeX, y + safeZomeY, 0, self.fontScale, self.fontScale)
    end
    
    
    --Draw page
    local offsetX = 1 --(self.width - font:getWidth(self.page[self.currentPage])*self.fontScale)/2
    local offsetY = 0--(self.height - font:getHeight(self.currentPage)*self.fontScale*(lines))/2
    
    local tx = self.x - self.width/2 + offsetX
    local ty = self.y - offsetY - lines*font:getHeight()*self.fontScale*0.5
    
    love.graphics.print(
      self.cutPage, 
      tx,
      ty,
      0,
      self.fontScale, 
      self.fontScale)
    
    if currentChar >= #self.currentPage then
      
      local x = self.x+self.width/2-20
      local y = self.y+70
      
      if gamepadMode then
        local scale = 4
        local width = font:getWidth(controls.interact.name)
        local height = font:getHeight()
        

        love.graphics.setColor(1,1,1)
        love.graphics.print(controls.interact.name, x + 2*smallTextureScale*texture.keypress:getWidth()/2, y + 2*smallTextureScale*texture.keypress:getHeight()/2, 0, scale, scale, width/2, height/2)
        
      else
        local a = (math.sin(love.timer.getTime()*6) + 1)*0.7
        love.graphics.setColor(1,1,1, a)
        love.graphics.displayKey(x, y, controls.interact.name, 2)
      end

    end
  end
  --lg.pop()
end


function DialogueBox:queueBook(filename)
  table.insert(self.bookQueue, filename)
  if not self.timeDisplayed then
    self:loadBook(filename)
  end
end


local function loadBook(name)
  
  if dialogue[name] then
    return coroutine.create(dialogue[name])
  else
    cout("Book '"..name.."' does not exist!")
  end
end


function DialogueBox:loadBook(name, speaker, pauseWorld)
  
  self.speaker = speaker
  
  self.bookCoroutine = speaker.bookCoroutine or loadBook(name)
  
  speaker.bookCoroutine = self.bookCoroutine
  
  if self.bookCoroutine then
    if pauseWorld ~= false then
      self.pauseWorld = true
    end
    
    
    self.displayed = true
    self.cutPage = ""
    
    self:nextPage()
    return true
  end
  return false
end


local function linesToString(lines)
  
  local s
  
  for i, line in ipairs(lines) do  
    if i == 1 then
      s = line
    else
      s = s.."\n"..line
    end
  end
  
  return s
end



function DialogueBox:nextPage()
  local active
  self.timeDisplayed = love.timer.getTime()
  
  
  local maxLines = 4
  
  if (not self.subPage) or self.subPage >= self.numberOfSubPages then
  
    --Get next page
    active, self.returnedDialogue = coroutine.resume(self.bookCoroutine, self.speaker)
    self.subPage = 1
    
    if self.returnedDialogue then
      local _, lines = font:getWrap(self.returnedDialogue, self.width/self.fontScale)
      self.numberOfSubPages = math.ceil(#lines/maxLines)
    end
  else
    
    self.subPage = self.subPage + 1
    
  end
  if self.returnedDialogue then
    self.currentPage = self:formatPage(self.returnedDialogue, self.subPage)
    self.speakerName = bookFunctions.getSpeaker()
  else
    self:closeBook()
  end
end


function DialogueBox:closeBook()
  self.pauseWorld = false
  self.displayed = false
  if coroutine.status(self.bookCoroutine) == "dead" then
    self.speaker.bookCoroutine = nil
  end
  self.bookCoroutine = nil
  self.currentPage = nil
  table.remove(self.bookQueue, 1)
  self.timeDisplayed = false
  if #self.bookQueue > 0 then
    self:loadBook(self.bookQueue[1])
  else
    camera.targetPoint = nil
    camera.targetZoom = nil
  end
  bookFunctions.setSpeaker()
end


function DialogueBox:formatPage(page, pageNumber)
  
  local _, remainderLines = font:getWrap(page, self.width/self.fontScale)
  local shownLines = {}
  
  
  local maxLines = 4
  
  for i = 1, maxLines do
    local targetLine = 1 + (pageNumber - 1)*maxLines
    table.insert(shownLines, remainderLines[targetLine])
    table.remove(remainderLines, targetLine)
  end
  
  local formattedString = linesToString(shownLines)
  local remainder       = linesToString(remainderLines)
  

  return formattedString, remainder
  
  
end


function DialogueBox.wordTableToString(table, i)
  local returnString = table[1]
  local remainder = nil

  for word = 2, i do
    returnString = returnString.." "..table[word]
  end
  
  
  if i ~= #table then
    remainder = table[i+1]
    
    for word = i+2, #table do
      remainder = remainder.." "..table[word]
    end
  end
  
  return returnString, remainder
end

local function countSub(string, substring)
  
  local _, count = string.gsub(string, substring, "")
  
  return count
  
end


function string.specialSubstring(inputString, index)
  
  local ignoredCharacters = {" ", "\n"}
  local normalSub = string.sub(inputString, 1, index)
  
  
  
  local ignoredCharacterCount = 0
  
  
  for i, ignoredCharacter in ipairs(ignoredCharacters) do
    ignoredCharacterCount = ignoredCharacterCount + countSub(normalSub, ignoredCharacter)
  end
  
  
  return string.sub(inputString, 1, index + ignoredCharacterCount)
  
end


--[[function string.specialSubstring(inputString, numberOfCharacters)
  
  local ignoredCharacters = {}
  local numberOfIgnoredCharacters = 0
  ignoredCharacters[" "] = true
  ignoredCharacters["\n"] = true
  
  for i = 1, #inputString do
    if numberOfCharacters < i then break end
    
    local byteoffset = utf8.offset(inputString, i) or 0
    
    if ignoredCharacters[string.sub(inputString, i - byteoffset, i - byteoffset)] then
      numberOfIgnoredCharacters = numberOfIgnoredCharacters + 1
    end
  end
  
  
  local cutPosition = numberOfCharacters + numberOfIgnoredCharacters
  
  
  local byteoffset = (utf8.offset(inputString, cutPosition+1) or 0) - 1
  
  return (string.sub(inputString, 1, byteoffset))
  
  
  
  
  
  
  
  

  --return string.sub(string, 1, cutPosition - byteoffset)
  
  --return string.sub(string, 1, cutPosition)
end]]






