local Menu = require("lib.classic"):extend()
local controlHandler = require "util.controlHandler"


function Menu:new()
  self.state = {}
  self.nodes = {}
  self.visible = false
  self:pushState("root")
  
  local function mousepressed(self, x, y, button, presses) 
    self:onClick(presses)
  end
  controlHandler:subscribeMousepressed(self, mousepressed)
  
end

function Menu:show()
  self.visible = true
end


function Menu:hide()
  self.visible = false
end


function Menu:toggle()
  self.visible = not self.visible
end


function Menu:pushState(state)
  table.insert(self.state, state)
end


function Menu:getState()
  return self.state[#self.state]
end


function Menu:popState()
  --Removes the top state layer, and returns it  
  return table.remove(self.state)
end


function Menu:setState(state)
  --Clears entire queue
  self.state = {state}
end


function Menu:getNodes()
  return self.nodes[self:getState()]
end


function Menu:onClick(presses)
  -- Call this when there is a mouse click
  if self.visible then
    if self:getNodes() then
      for i, node in ipairs(self:getNodes()) do
        node:checkClick()
      end
    else
      cout("No nodes!")
    end
  end
end


function Menu:onRelease()
  -- Call this when there is a mouse click
  
  for i, node in ipairs(self:getNodes()) do
    if node.onRelease then
      node:onRelease()
    end
  end
  
end


function Menu:draw()
  love.graphics.push()
  
  love.graphics.translate(love.graphics.getWidth()/2, love.graphics.getHeight()/2)
  love.graphics.scale(love.graphics.getHeight()/720)
  love.graphics.translate(-love.graphics.getWidth()/2, -love.graphics.getHeight()/2)

  
  if self.visible then
    if self.nodes[self:getState()] then
      for i, node in ipairs(self:getNodes()) do
        node:draw()
      end
    end
  end
  love.graphics.pop()
end

--Pop and push functions automatically given to nodes if none other is given
local function nodePopState(self)
  self.parentMenu:popState()
end

local function nodePushState(self, state)
  self.parentMenu:pushState(state)
end


function Menu:addNode(node)
  
  if not node.requiredState then
    error("Node was not given a required state!", 2)
  end
  
  if not node.checkClick then
    error("Node was not given a click check function!", 2)
  end
  
  if not node.draw then
    error("Node was not given a draw function!", 2)
  end
  
  
  node.parentMenu = self
  
  node.popState = nodePopState
  node.pushState = nodePushState
  
  
  
  self.nodes[node.requiredState] = self.nodes[node.requiredState] or {}
  table.insert(self.nodes[node.requiredState], node)
  
end


return Menu

