local Menu = require "load.menu"
local passes = require "rendering"

local pauseMenu = Menu()


local function nodeDraw(self)
  
  local x = self.x
  local y = self.y
  local width = self.width
  local height = self.height
  
  if self.toggled then
    if self:checkHover() then
      love.graphics.setColor(0, 0.8, 0.8)
    else
      love.graphics.setColor(0, 1, 1)
    end
  else
    if self:checkHover() then
      love.graphics.setColor(0.8, 0.8, 0.8)
    else
      love.graphics.setColor(1,1,1)
    end
  end
  love.graphics.rectangle("fill", x - width/2, y - height/2, width, height)
  love.graphics.setLineWidth(5)
  love.graphics.setColor(0.7,0.7,0.7)
  love.graphics.rectangle("line", x - width/2, y - height/2, width, height)
  love.graphics.setLineWidth(1)
  love.graphics.setColor(0,0,0)
  love.graphics.print(self.label, x, y, 0, 2, 2, font:getWidth(self.label)/2, font:getHeight(self.label)/2)
end


local function nodeCheckHover(self)
  local screenScale = love.graphics.getHeight()/720
  local x = self.x
  local y = self.y
  local width = self.width
  local height = self.height
  
  local mx, my = love.mouse.getPosition()
  
  
  --Transform to fit screen scale
  mx = (mx - love.graphics.getWidth()/2)/screenScale + love.graphics.getWidth()/2
  my = (my - love.graphics.getHeight()/2)/screenScale + love.graphics.getHeight()/2
  

  
  
  return x - width/2 <= mx and 
     mx < x + width/2 and
     y - height/2 <= my and 
     my < y + height/2
end


local function nodeCheckClick(self)
  
  if self:checkHover() then
    if self.onClick then
      self:onClick()
    end
  end
end


local function createNode(x, y, label, state, onClickFunction, toggled)
  local self = {}
  
  self.x = x
  self.y = y
  self.width = 300
  self.height = 100
  self.label = label
  self.draw = nodeDraw
  self.checkClick = nodeCheckClick
  self.onClick = onClickFunction
  self.checkHover = nodeCheckHover
  self.requiredState = state or "root"
  --Not required
  self.toggled = toggled
  
  return self
end

local function getY(pos, groupNumber)
  pos = pos - 1
  
  local separation = 150
  
  
  return love.graphics.getHeight()/2 + separation * (pos - ((groupNumber-1)/2))
  
end

local x = love.graphics.getWidth()/2


pauseMenu:addNode(createNode(x, getY(1, 3), "Resume",  nil, function(self) self.parentMenu:hide() end))
pauseMenu:addNode(createNode(x, getY(2, 3), "Options", nil, function(self) self:pushState("options") end))
pauseMenu:addNode(createNode(x, getY(3, 3), "Quit",    nil, function(self) love.event.quit() end ))


pauseMenu:addNode(createNode(x, getY(1, 3), "Game", "options", function(self) self:pushState("gameOptions") end))
pauseMenu:addNode(createNode(x, getY(2, 3), "Graphics", "options", function(self) self:pushState("graphicsOptions") end))
pauseMenu:addNode(createNode(x, getY(3, 3), "Audio", "options", function(self) self:pushState("audioOptions") end))



pauseMenu:addNode(createNode(x, getY(1, 1), "Flight Assist", "gameOptions", 
    function(self) 
      settings.game.flightAssist = not settings.game.flightAssist
      self.toggled = settings.game.flightAssist
      if settings.game.flightAssist then
        cout("Flight assist on")
      else
        cout("Flight assist off")
      end
    end
  ))

pauseMenu:addNode(createNode(x, getY(1, 1), "IOU Audio Options", "audioOptions", function(self) cout("Beep") end))


pauseMenu:addNode(createNode(x, getY(1, 1), "Compatibility Mode", "graphicsOptions", 
    function(self) 

      
      passes:createCanvases(love.graphics.getWidth(), love.graphics.getHeight())
      
      settings.graphics.lightingCompatibility = not settings.graphics.lightingCompatibility
      self.toggled = settings.graphics.lightingCompatibility
      
      if settings.graphics.lightingCompatibility then
        cout("Canvas compatibility on (This setting will be slightly faster when off for supported machines)")
      else
        cout("Canvas compatibility off (This setting will be slightly faster when off for supported machines)")
      end
      
    end, settings.graphics.lightingCompatibility))

pauseMenu:addNode(createNode(x, getY(0, 1), "Vsync", "graphicsOptions", 
    function(self) 

      if love.window.getVSync() == 1 then
        love.window.setVSync(0)
      else
        love.window.setVSync(1)
      end
      
      self.toggled = love.window.getVSync() == 1

      
    end, love.window.getVSync() == 1))


pauseMenu:addNode(createNode(x, getY(2, 1), "Fast Lighting", "graphicsOptions", 
    function(self) 

      settings.graphics.fastLighting = not settings.graphics.fastLighting
      
      self.toggled = settings.graphics.fastLighting

      
    end, settings.graphics.fastLighting))


function pauseMenu:update(dt)
  if controls.menuUp.thisFrame then
    self:up()
  elseif controls.menuDown.thisFrame then
    self:down()
  elseif controls.menuEnter.thisFrame then
    self:enter()
  end
end


return pauseMenu