local programVersion = "Demo 0.1.16"

local demoMode = true
local startPaused = demoMode
local paused = false
onStartMenu = demoMode

settings           = require "gameSettings"
local pauseManager = require "util.pauseManager"
local pixelCanvas  = require "lib.pixelCanvas"
local screenFader  = require "UI.screenFader"
local volume       = require "volumeHandler"
local controlHandler = require "util.controlHandler"
local pauseMenu
local musicVolume = -0.1

gamepadMode = false


if demoMode then
  programVersion = "Public ".. programVersion
end


local analyseFramerate = false
local framerateResults = {}
local framerateObjectCount = {}
local memoryUseageAverage = 0

builderEnabled = true -- Lock down the engine if needed

currentFrame = 0
local clickedThisFrameChecker = {}
clickedThisFrameChecker.primary = false
clickedThisFrameChecker.secondary = false
clickedThisFrame = {}
clickedThisFrame.primary = false
clickedThisFrame.secondary = false

scene = {}
scene.lightHarshness = 0.1

textureScale = 5

skyColour = {132/255, 173/255, 1}

--skyColour = {255/255, 243/255, 212/255}


window = {}
mouse = {}
mouse.x = 0
mouse.y = 0
mouse.worldX = 0
mouse.worldY = 0
mouse.isDown = {} 
mouse.lastMoved = 0 
controls = {}

flightJoystick = {}
flightJoystick.x = 0
flightJoystick.y = 0


--Table for which order things should be drawn in
local tableNumber = 0
require "util.miscFunctions"
require "lib.loadingAnalyser"
local startMenu
local passes = require "rendering"


require "load.shaders"
loadShaders()
local lightHandler = require "lighting.lightHandler"

passes:setAmbient(1)


local sortTable = 
{
  "camera",
  "textEditor",
  "dialogueBox",
  "logicProp",
  "directionParticles",
  "grass",
  "firefly",
  "sword",
  "flowLevelPolygon",
  "levelPolygon",
  "darkZone",
  "waterBlock",
  "pixelCanvas",
  "fireball",
  "lingerLight",
  "sparks",
  "raindrops",
  "bubble",
  "player",
  "squeeze",
  "rainEmitter",
  "particleGen",
  "tempAnim",
  "slidingDoor",
  "door",
  "dragonCore",
  "physicsCube",
  "flow",
  "fireEmitter",
  "interactable",
  "button",
  "talkative",
  "textProp",
  "brazier",
  "lightEmitter",
  "crumblePixel",
  "prop",
  "trails",
  "levelLoader",
  "directionStone",
  "backgroundPolygon",
  "cloudSheet",
  "other"
}

sortTable = swapKeyValues(sortTable)

local targetFps = 60
local waterPixelCanvas
local trailPixelCanvas


function love.load()  
  love.graphics.setDefaultFilter("nearest", "nearest", 4) 
  debugMode = false
  screenshotMode = false
  builderMode = false

  drawTable = {}
  darkZoneTable = {}
  --table to help find named entities
  nameIndex = require "nameIndex"
  require "util.consoleOutput"

  
  require "animationPlayer"
  require "lighting.normal"
  texture, imageData = require "load.textures"
  require "load.sounds"

  love.window.setMode(0,0)
  love.window.setFullscreen(demoMode)
  
  
  math.randomseed(os.time())
  
  passes:createCanvases(love.graphics.getWidth(), love.graphics.getHeight())
  
  
  lume = require "lib.lume"
  utf8 = require "utf8"
  
  require "gameSettings"
  
  require "lib.blueGraphics"
  require "lib.vector"
  require "lib.line"
  require "lib.pixpol"
  require "load.events"
  require "point"
  require "textEditor"
  require "attributeEditor"
  require "entities.entity"
  require "entities.physicsItem"
  require "entities.dragonCore"
  require "entities.prop"
  require "entities.builder.builder"
  require "entities.polygon"
  require "entities.physicsCube"
  require "entities.levelPolygon"
  require "entities.darkZone"
  require "entities.cloudSheet"
  require "entities.grass"
  require "entities.particleGenerator"
  require "entities.firefly"
  require "entities.tempAnim"
  require "entities.projectile"
  require "entities.fireball"
  require "entities.sparks"
  require "entities.raindrops"
  require "entities.particles.bubble"
  require "entities.waterBlock"
  require "entities.textProp"
  require "entities.trailRibbon"
  
  require "entities.enemies.flow"

  require "entities.special props.logicProp"
  require "entities.special props.lightEmitter"
  require "entities.special props.fireEmitter"

  require "entities.interactables.interactable"
  require "entities.interactables.special interactables"
  require "entities.interactables.talkative"
  require "entities.interactables.door"
  require "entities.interactables.slidingDoor"
  require "entities.interactables.brazier"
  require "entities.interactables.fireLantern"
  require "entities.interactables.directionStone"
  require "entities.interactables.coreHolder"
  require "entities.interactables.squeeze"
  require "entities.interactables.levelLoader"

  require "entities.special props.rainEmitter"

  require "load.dialogue"
  require "load.level"
  require "util.polygonTest"
  require "entities.player.player"
  

  
  require "startMenuButton"
  
  startMenu = require "startMenu"
  

  local function water_custom_draw(self)
    love.graphics.setColor(0.3,0.56,1,0.5)
    love.graphics.draw(self.canvas, 0, 0, 0)
  
  end
  
  
  waterPixelCanvas = pixelCanvas.create("waterBlock", water_custom_draw)
  trailPixelCanvas = require "pixelCanvases.trail"
  
  
  
  sound.music.hopesAndDreams:setVolume(0)

  smallTextureScale = 40/(texture.drea_idle.main:getHeight() - 2)

  -- Basic physics code based on love2D tutorial
  -- the height of a meter in our world will be 64px

  love.physics.setMeter(64)
  

  currentLevel = Level(nil)

  --[[pauseMenu = Menu("Pause Menu", 
    {
      MenuElement("Resume"), 
      MenuElement("Settings"), 
      MenuElement("Quit", love.event.quit)
    })]]
  
  local gravity = 9.81*love.physics.getMeter()
  
  world = love.physics.newWorld(0, gravity, true)
  require "mainPhysicsCallbacks"
  player1 = Player(-10460, -2650)
  builder = Builder(0,0)
  camera = require "entities.camera"
  createSparkGroup()
  createRaindropGroup()

  

  
  love.keyboard.setKeyRepeat(true)

  
  loadingScreen()
  
  
  --love.window.setVSync(settings.graphics.vsync or 1)

  
  
  love.mouse.setVisible(false)
  window.width = love.graphics.getWidth()
  window.height = love.graphics.getHeight()


  font = love.graphics.newImageFont("textures/font.png", 
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"..
    "abcdefghijklmnopqrstuvwxyz"..
    "0123456789().,-=+\"':;/?! *_|@ⒶⒷⓎⓧ", 1)

  love.graphics.setFont(font)
  
  textEditor = TextEditor()
  dialogueBox = DialogueBox(window.width/2, window.height*6.8/8, nil)
  
  cloudTable = {}
  

  for i = 0, 3 do

    love.math.setRandomSeed(i)
    local layerDepth = i*5 + 1.0
    table.insert(cloudTable, CloudSheet(love.math.random(-100000, 100000), love.math.random(-100000, 100000), layerDepth))    

  end


  
  currentLevel:loadLevel("overworld")
  player1:spawn("sp_origin")
  startMenu:make()
  
  
  
  if (not demoMode) and onStartMenu then
    startMenu:showMenu()
  end
  
  
  if not onStartMenu then
    startGame()
  end
  pauseMenu = require "load.pauseMenu"
end


function startGame()

  love.mouse.setPosition(window.width/2, window.height/2)
  if demoMode then
    sound.music.hopesAndDreams:play()
  end
  lightHandler:sendLights()
end




function love.update(dt)
  
  
  
  local numberOfUpdatesPerFrame = 1
  dt = dt / numberOfUpdatesPerFrame
  if (startPaused and demoMode) then
    --Intro message update
    if not pauseMenu.visible then
      manageControls()
    end
    
    if controls.interact.isDown then
      startPaused = false
      love.mouse.setPosition(window.width/2, window.height/2)
      sound.music.hopesAndDreams:seek(0.67)
      startMenu:showMenu()

    end
  elseif onStartMenu then
    --Start menu update
    manageControls()
    screenFader:update(dt)
    startMenu:update(dt)
  else
    
    for i = 1, numberOfUpdatesPerFrame do
      currentFrame = currentFrame + 1
      local j = 0

      manageControls()

      
      screenFader:update(dt)




      if musicVolume < 1 then
        musicVolume = musicVolume + dt*0.5
        sound.music.hopesAndDreams:setVolume(musicVolume * volume.getMusic())
      elseif musicVolume > 1 then
        musicVolume = 1
        sound.music.hopesAndDreams:setVolume(1 * volume.getMusic())
      end
      mouse.lastX = mouse.x
      mouse.lastY = mouse.y 



      


      if gamepadMode then
        local joystick = joysticks[1]
        flightJoystick.x = joystick:getGamepadAxis("leftx")
        flightJoystick.y = joystick:getGamepadAxis("lefty")
      end
      
      mouse.x = love.mouse.getX() - window.width/2
      mouse.y = love.mouse.getY() - window.height/2


      mouse.worldX = mouse.x/camera:getFinalZoom() + camera.x
      mouse.worldY = mouse.y/camera:getFinalZoom() + camera.y




      if not (mouse.x == mouse.lastX and mouse.y == mouse.lastY) then
        mouse.lastMoved = love.timer.getTime()
      end

      if paused then
        
        pauseMenu:update(dt)

      else
        if builderMode then

          if not (textEditor.active or pauseMenu.visible) then
            builder:update(dt)
          end
        else
          
          dt = math.min(dt, 1/30)
          
          
          if not pauseMenu.visible then
            if not dialogueBox.pauseWorld then
              player1:clearCollisionTable()
            end
            
            
            local velocityiterations = 8
            local positioniterations = 4
            world:update(dt, velocityiterations, positioniterations)
            camera:update(dt)
         

            
            for i, cloud in ipairs(cloudTable) do
              cloud:update(dt)
            end
            
            
          
            for i, object in ipairs(drawTable) do
       
              if object.update ~= nil and not dialogueBox.pauseWorld then
                -- Only update if current stage. If not, set as inactive
                if object.activeStage == currentLevel.stage or object.activeStage == nil then

                  --Update and set active
                  if object.body and (not object.body:isDestroyed()) then
                    object.body:setActive(true)
                  end
                  object:update(dt)
                  

                  
                  
                else
                  
                  if object.body and (not object.body:isDestroyed()) then
                    object.body:setActive(false)
                  end
                end
              end
              if object.updatePaused then
                --Update even if in a dialogue
                object:updatePaused(dt)
              end
            end
            
            flushDrawTable()
          end
          
        end
      end
      
      
      
      
      
    end
  end
  
  
end



function flushDrawTable()
  --Remove unwanted objects if tagged with 'remove'
  for i, object in ipairs(drawTable) do
    if object.remove then

      if object.attributes and object.attributes.name and object.attributes.name ~= "" then
        nameIndex:removeEntity(object)
      end

      table.remove(drawTable, i)
    end
  end
end


local function sortDrawTable()
  table.sort(drawTable, 
    function(k1, k2)
      --True if k2 should be in front of k1


      local k1z = k1.z or 1
      local k2z = k2.z or 1

      --Objects with a lower Z will always be returned first
      if k1z ~= k2z then
        return k1z > k2z
      end

      local k1Type = k1.sortType or k1.type or "other"
      local k2Type = k2.sortType or k2.type or "other"
      
      
      if (not sortTable[k1Type]) and k1.isProp then
        k1Type = "prop"
      end
      
      if (not sortTable[k2Type]) and k2.isProp then
        k2Type = "prop"
      end
      

      if sortTable[k1Type] == nil then
        error(k1Type .." does not exist in the sort table!")
      end
      if sortTable[k2Type] == nil then
        error(k2Type .." does not exist in the sort table!")
      end

      if k1Type == "levelPolygon" and k2Type == "levelPolygon" then
        if k1 == builder.selectedPolygon or k1 == builder.activePolygon then
          return false
        end
        if k2 == builder.selectedPolygon or k2 == builder.activePolygon then
          return true
        end


        if k1.attributes.isBackground then
          return true
        end
        if k2.attributes.isBackground then
          return false
        end

      end


      if k1Type == k2Type then
        if k1.attributes and k2.attributes and k1.attributes.drawPriority and k2.attributes.drawPriority then
          return k1.attributes.drawPriority < k2.attributes.drawPriority
        end
      end


      return sortTable[k1Type] > sortTable[k2Type]
    end
  )

end

local firstDraw = true

function love.draw()
  
  if startPaused and demoMode then

    love.graphics.setBackgroundColor(0,0,0)
    local scale = 2.5 * (window.height/1080) --Adjust for smaller screens
    local safeZone = 100
    love.graphics.setColor(1,1,1)
    love.graphics.printf(
      [[Hello there! This is a demo of my game, thanks for checking it out!
      
      If you want help me develop this further, sending me a video of you playing would be a great help!
      
      I also have a YouTube channel at www.youtube.com/@bobble68, which I *sometimes* upload update videos to.
      
      - Bobble
      
      
      Press ]]..controls.interact.name.." to start", safeZone, safeZone, (love.graphics.getWidth()-2*safeZone)/scale, "center", 0, scale, scale)
    
  elseif onStartMenu then
    
      startMenu:draw()
      screenFader:draw()
      
  else

    love.graphics.setColor(1,1,1,1)

    sortDrawTable()
    
    --Draw to pixel canvases
    
    waterPixelCanvas:drawTo()
    trailPixelCanvas:drawTo()
    
    --Draw ambient lighting
    
    love.graphics.setCanvas(passes.ambient)
    
    
    for i, shadowCaster in ipairs(require "tables.shadowCasters") do
      shadowCaster:drawShadow()
    end
    
    for i, darkZone in ipairs(darkZoneTable) do
      darkZone:drawAmbient()
    end
    
    
    --Main draw loop
    love.graphics.setCanvas(passes.colour, passes.depth, passes.normal)
    love.graphics.setShader(shader.renderPasses)
    for i, v in ipairs(drawTable) do
      
      if (not v.isActive) or v:isActive() then

        local renderStart = nil
        if analyseFramerate then
          renderStart = love.timer.getTime()
        end
        
        if v.draw then
          blue.graphics.setZ(0)
          v:draw()
        end
        
        
        if analyseFramerate then
          if framerateResults[v.type] then
            framerateResults[v.type] = framerateResults[v.type] + love.timer.getTime() - renderStart   
            framerateObjectCount[v.type] = framerateObjectCount[v.type] + 1 
          else
            framerateResults[v.type] = love.timer.getTime() - renderStart
            framerateObjectCount[v.type] = 1
          end
        end
        
      end
    end
    
    
    
    love.graphics.setShader()
    love.graphics.setCanvas()
    

    passes:render()
    
    
    
    for i, v in ipairs(drawTable) do
      if v.activeStage == currentLevel.stage or v.activeStage == nil then
        if v.drawOverlay then
          v:drawOverlay()
        end
      end
    end
    
    screenFader:draw()

    --printInstructions()
    textEditor:draw()
    pauseMenu:draw()
    if not builderMode then
      love.graphics.setColor(1,1,1)
    else
      builder:draw()
      love.graphics.setColor(0.5,0.9,1,0.3)
    end

    if not screenshotMode then
      --Draw mouse


      if love.timer.getTime() - mouse.lastMoved < 3 then
        love.graphics.circle("fill", mouse.x + window.width/2, mouse.y + window.height/2, 5)
        love.graphics.setColor(0,0,0)
        love.graphics.circle("line", mouse.x + window.width/2, mouse.y + window.height/2, 5)
      end
      love.graphics.setColor(1, 0, 0)



      --blue.graphics.circle("fill", mouse.x - camera.x, mouse.y - camera.y, 5)
      love.graphics.setColor(0,0,0)
      --blue.graphics.circle("line", mouse.x - camera.x, mouse.y - camera.y, 5)

      --blue.graphics.circle("fill", 0,0, 100)

 

      if builderMode then
        blue.graphics.printShadow(
          "fps:  "     ..love.timer.getFPS( )..
          "\nx:  "     ..(math.floor(mouse.worldX*100)/100)..
          "\ny:  "     ..(math.floor(mouse.worldY*100)/100)..
          "\nZoom:  "  ..(math.floor(camera.zoom*100)/100)..
          "\nTester:  "..tostring(tester)..
          "\nMode:  "  ..string.upper(builder.mode), 10,10)

      elseif debugMode then
      
        local s = "fps:  "   .. love.timer.getFPS( )..
                  "\nv:  "   ..math.floor(vector.getMagnitude(velocityVector)/love.physics.getMeter()) .. "m/s"..
                  "\nZoom:  "..(math.floor(camera.zoom*1000)/1000)..
                  "\nMemory useage:  "..math.floor(collectgarbage('count')/10)*10 .. " kb"..
                  "\nTexture Memory useage:  "..math.ceil(love.graphics.getStats().texturememory/1048576).. "mb"
                  
        
        if tester then
          s = s.."\nTester:  "..tostring(tester)
        end
        
      
        blue.graphics.printShadow(s, 10,10)
          

      end
      console:draw()
      love.graphics.setColor(0,0,0)

      blue.graphics.printShadow(programVersion, 10, math.floor(window.height)-20) 
      love.graphics.setColor(1,1,1)
    end




    --glowCanvas:renderTo(love.graphics.clear)
  end
  
  --[[for i, v in pairs(love.graphics.getStats()) do
    print(i, v)
  end]]

end

function printInstructions()
  love.graphics.setColor(1,1,1)
  local xOffset, yOffset = blue.standardTransform(-10400, -2750)

  love.graphics.print(
    [[
      Use left and right
      or A and D to run
      ]], xOffset, yOffset, 0, camera.zoom)

  xOffset, yOffset = blue.standardTransform(-9000, -2850)
  love.graphics.print(
    [[
      Hold shift to
      tuck into a ball
      ]], xOffset, yOffset, 0, camera.zoom)

  xOffset, yOffset = blue.standardTransform(-7550, -2350)
  love.graphics.print(
    [[
      Point with your mouse
      and press space to fly
      ]], xOffset, yOffset, 0, camera.zoom)
end




local waterTest = require "events"


local function keypressed(self, key, scancode, isrepeat )
  


  gamepadMode = false
  
  if key == "escape" and not builder.showBuilderMenu then
    pauseMenu:toggle()
    pauseMenu:setState("root")
  end


  if not startPaused then
    if (not textEditor.active) and (not pauseMenu.visible) then
      
      if key == "escape" and builder.showBuilderMenu then
        builder:hideMenu()
      elseif key ==  "\\" or key == "f3"  then
        debugMode = not debugMode

      elseif key == "f1" then
        screenshotMode = not screenshotMode

      elseif builderEnabled then

        if key == "tab" then
          if builderMode and not (builder.selectedObject and builder:isModeSelectable()) then
            builder:toggleMenu()
          end

        elseif not builder.showBuilderMenu then


          if key ==  "b" then
            screenshotMode = false
            builderMode = not builderMode
            builder:hideMenu()
            if builder.activePolygon then
              builder.activePolygon:delete()
            end
            if builderMode then
              builder:setPosition(-camera.x, -camera.y)
            else
              camera:setPosition(-builder.x, -builder.y)
            end
          
          elseif key == "s" and love.keyboard.isScancodeDown("lctrl") and love.keyboard.isScancodeDown("lshift") then
            
            textEditor:show(currentLevel.currentLevelName, 
              function()
                local filename = textEditor.value
                
                currentLevel:saveLevel(filename)
                cout("Saved as ".. filename)
              end)
            
          elseif (key == "f5" or (key == "s" and love.keyboard.isScancodeDown("lctrl"))) and not isrepeat then
            local name = currentLevel:saveLevel()
            cout("Saved as "..name)

          elseif key == "f2" then
            currentLevel:deload()
            cout("Cleared world!")

          elseif key == "f8" and not isrepeat then
            currentLevel:loadLevel("overworld")
            builder.selectedObject = nil
            cout("Loaded baked level")
          elseif key == "/"  then

            cout("Select Stage")
            textEditor.callback = function()
              currentLevel:changeLevel(textEditor.value)
            end

            textEditor:show(currentLevel.stage)
            textEditor.value = currentLevel.stage

          end
        end

      end



    else
      --Text editor controls
      if key == "backspace" then
        textEditor.value = string.sub(textEditor.value, 1, textEditor.cursorPos-1).. string.sub(textEditor.value, textEditor.cursorPos+1, #textEditor.value)
        textEditor.cursorPos = math.max(textEditor.cursorPos - 1, 0)
      elseif key == "delete" then
        textEditor.value = string.sub(textEditor.value, 1, textEditor.cursorPos).. string.sub(textEditor.value, textEditor.cursorPos+2, #textEditor.value)
      elseif key == "return" then
        if love.keyboard.isScancodeDown("lshift") then
          textEditor:insertText("\n")
        else
          textEditor:finish()
        end
      elseif key == "escape" then
        textEditor:close()
      elseif key == "right" then
        textEditor.cursorPos = math.min(textEditor.cursorPos + 1, #textEditor.value)
      elseif key == "left" then
        textEditor.cursorPos = math.max(textEditor.cursorPos - 1, 0)
      elseif key == "v" and (not clipboard) and love.keyboard.isScancodeDown("lctrl") then
        local clipboard = love.system.getClipboardText( )
        textEditor.value = string.sub(textEditor.value, 1, textEditor.cursorPos)..clipboard.. string.sub(textEditor.value, textEditor.cursorPos+1, #textEditor.value)
        textEditor.cursorPos = textEditor.cursorPos + #clipboard

      end
    end
  end
end

controlHandler:subscribeKeypressed({}, keypressed)

function love.joystickadded()
  joysticks = love.joystick.getJoysticks()
end 


function love.joystickpressed(joystick, button)
  gamepadMode = true
 
end


function love.joystickaxis()
  gamepadMode = true
end



function loadingScreen()

  local scale = love.graphics.getHeight()/texture.drea_tucked.main:getHeight()*0.1
  local r = love.timer.getTime()*4
  love.graphics.clear()
  love.graphics.setColor(1,1,1)
  love.graphics.draw(texture.drea_tucked.main,
    love.graphics.getWidth() - texture.drea_tucked.main:getWidth()*scale,
    love.graphics.getHeight() - texture.drea_tucked.main:getHeight()*scale,
    r, scale, scale,
    texture.drea_tucked.main:getWidth()/2,
    texture.drea_tucked.main:getHeight()/2
  )


  love.graphics.present()

end




function manageControls()
  --[[
  Here you can remap the controls. The first argument refers to the action and should be left unchanged, the second 
  is the condition that needs to be met to activate the control, and the third argument is what it will be referred to 
  as in game.
  ]]
  
  local joystick
  
  local gamepadWalkRight
  local gamepadWalkLeft
  
  if gamepadMode and joysticks then
    joystick =  joysticks[1]
    local safeZone = 0.5
    
    local inSafeZone = joystick:getGamepadAxis("leftx")^2 + joystick:getGamepadAxis("lefty")^2 < safeZone^2
    
    gamepadWalkRight = joystick:getGamepadAxis("leftx") > 0 and not inSafeZone
    gamepadWalkLeft = joystick:getGamepadAxis("leftx") < 0 and not inSafeZone
  end
  
  local lTriggerIsDown = (joystick and joystick:getGamepadAxis("triggerleft") > 0.1)
  local rTriggerIsDown = (joystick and joystick:getGamepadAxis("triggerright") > 0.1)
  local gamepadAIsDown = (joystick and joystick:isGamepadDown("a"))
  local gamepadXIsDown = (joystick and joystick:isGamepadDown("x"))
  local gamepadYIsDown = (joystick and joystick:isGamepadDown("y"))
  local gamepadBIsDown = (joystick and joystick:isGamepadDown("b"))
  local gamepadLBIsDown = (joystick and joystick:isGamepadDown("leftshoulder"))
  

  
  
  controlController("walkRight",     love.keyboard.isScancodeDown("right", "d") or gamepadWalkRight, "D")
  controlController("walkLeft",      love.keyboard.isScancodeDown("left", "a")  or gamepadWalkLeft,  "A")
  
  controlController("fireball",      love.mouse.isDown(1) or gamepadLBIsDown,  "Left Click")
  
  controlController("interact",      love.keyboard.isDown("f") or gamepadAIsDown, "F", "Ⓐ")
  controlController("flap",          love.keyboard.isScancodeDown("space") or lTriggerIsDown,  "space")
  controlController("tuck",          love.keyboard.isScancodeDown("lshift") or love.keyboard.isScancodeDown("rshift") or rTriggerIsDown, "left shift")
  controlController("timeBubble",    love.keyboard.isDown("1") or gamepadYIsDown, "1", "Ⓨ")
  controlController("lockDirection", love.keyboard.isScancodeDown("lalt"), "left alt")

  controlController("menuLeft",      love.keyboard.isDown("q"),   "Q")
  controlController("menuRight",     love.keyboard.isDown("e"),   "E")
  controlController("menuUp",        love.keyboard.isDown("w") or love.keyboard.isDown("up"),   "W")
  controlController("menuDown",      love.keyboard.isDown("s") or love.keyboard.isDown("down"), "S")
  controlController("menuEnter",     love.keyboard.isDown("return"),  "Enter")
  
  
  
  
  clickedThisFrame.primary = false
  clickedThisFrame.secondary = false

  if love.mouse.isDown(1) then
    if not clickedThisFrameChecker.primary then
      clickedThisFrame.primary = true
    end
    clickedThisFrameChecker.primary = true
  else
    clickedThisFrameChecker.primary = false
  end

  if love.mouse.isDown(2) then
    if not clickedThisFrameChecker.secondary then
      clickedThisFrame.secondary = true
    end
    clickedThisFrameChecker.secondary = true
  else
    clickedThisFrameChecker.secondary = false
  end
  
  
  
  
  
end


function controlController(name, isDown, buttonName, gamepadButtonName)
  if not controls[name] then
    controls[name] = {}
  end
  
  if gamepadMode and gamepadButtonName then
    controls[name].name = gamepadButtonName
  else
    controls[name].name = buttonName
  end
  if isDown then
    controls[name].thisFrame = not controls[name].isDown
    controls[name].upThisFrame = false
    controls[name].isDown = true
  else
    controls[name].upThisFrame = controls[name].isDown
    controls[name].isDown = false
    controls[name].thisFrame = false
  end
end


function love.resize( width, height )
  window.width = width
  window.height = height
end



function love.quit(a)
  console:dumpOutput()
  
  if analyseFramerate then
    local normalise = false
    local totalRenderTime = 0

    for i, v in pairs(framerateResults) do
      if normalise then
        totalRenderTime = totalRenderTime + v/framerateObjectCount[i]
      else
        totalRenderTime = totalRenderTime + v
      end

    end

    if not normalise then
      for i, v in pairs(framerateResults) do
        print(i, math.floor(v/totalRenderTime*100000)/1000 .."%" )
      end
    else
      for i, v in pairs(framerateResults) do
        print(i, math.floor(((v/framerateObjectCount[i])/totalRenderTime*100000)/1000) .."%" )
      end
    end
  end
end




tmpPrint = print
