local passes = {}

local intermediate
local intermediate2

local lightHandler = require "lighting.lightHandler"



local function boxBlur(canvas, radius)
  shader.blurX:send("kernalRadius", radius)
  shader.blurY:send("kernalRadius", radius)
  
  
  love.graphics.setCanvas(intermediate)
  
  love.graphics.clear()
  love.graphics.setShader(shader.blurX)
  love.graphics.draw(canvas)
  
 
  love.graphics.setCanvas(canvas)
  love.graphics.setShader(shader.blurY)
  love.graphics.draw(intermediate)
  love.graphics.setShader()
  love.graphics.setCanvas()
  
end


local downSampleCanvas 

local function blurCanvas(canvas, radius)
  love.graphics.setColor(1,1,1)
  
  love.graphics.setCanvas(downSampleCanvas)
  love.graphics.clear(0,0.25,0)
  local scale = downSampleCanvas:getWidth()/canvas:getWidth()
  love.graphics.draw(canvas, 0, 0, 0, scale)
  love.graphics.setCanvas()
  

  
  love.graphics.setCanvas(canvas)

  love.graphics.draw(downSampleCanvas, 0, 0, 0, 1/scale)
  
  love.graphics.setCanvas()
  
  boxBlur(canvas, radius)

end



local aColour = {1,1,1}

function passes:createCanvases(width, height)
  
  
  self.colour      = love.graphics.newCanvas(width, height)
  self.normal      = love.graphics.newCanvas(width, height)
  --self.bloom   = love.graphics.newCanvas(width, height)
  self.overlay     = love.graphics.newCanvas(width, height)
  self.scene       = love.graphics.newCanvas(width, height)
  self.final       = love.graphics.newCanvas(width, height)
  self.background  = love.graphics.newCanvas(width, height)
  self.ambient     = love.graphics.newCanvas(width, height)
  self.ambient:setFilter("linear")
  intermediate     = love.graphics.newCanvas(width, height)
  downSampleCanvas = love.graphics.newCanvas(width*0.25, height*0.25, {msaa = 16})
  downSampleCanvas:setFilter("linear")
  
  
  
  local supportedformats = love.graphics.getCanvasFormats()

  if settings.graphics.lightingCompatibility or true then
    if supportedformats["rgba16f"] then
      self.depth = love.graphics.newCanvas(width, height, {format  = "rgba16f"})
    else
      cout("WARNING - This graphics driver does not support the rgba16f format, some things may not draw correctly.")
      self.depth = love.graphics.newCanvas(width, height)
    end
  else
    if supportedformats["r16f"] then
      self.depth = love.graphics.newCanvas(width, height, {format  = "r16f"})
    else
      cout("WARNING - This graphics driver does not support the r16f format, some things may not draw correctly.")
      self.depth = love.graphics.newCanvas(width, height)
    end

  end
  
  
  love.graphics.setCanvas(self.ambient)
  love.graphics.clear(aColour)
  love.graphics.setCanvas()
  
  
end

--This is easier
function passes:setAmbient(...)
  lightHandler:setAmbient(...)
end

function passes:getAmbient()
  return lightHandler:getAmbient()
end


local canvasLimit = love.graphics.getSystemLimits( ).multicanvas
function passes:clearCanvases()
  
  local canvases = {passes.colour, passes.background, passes.normal,   passes.depth, passes.ambient, passes.scene, self.shadowCasters} 
  local canvasColours = {{0,0,0,0}, skyColour, {0.5, 0.5, 1}, {-65504, 0, 0}, self:getAmbient(), {0,0,0,0}, {0,0,0,0}}
  
  local segments = {}
  local segmentColourList = {}

  
  if math.ceil(#canvases/canvasLimit) == 1 then
    segments = {canvases}
    segmentColourList = {canvasColours}
    
    love.graphics.setCanvas(unpack(canvases))
    love.graphics.clear(unpack(canvasColours))
    
  else
    for i, canvas in ipairs(canvases) do
      local j = math.ceil(i/canvasLimit)
      segments[j] = segments[j] or {}
      segmentColourList[j] = segmentColourList[j]  or {}
      table.insert(segments[j], canvas)
      table.insert(segmentColourList[j], canvasColours[i])
    end
    
    for i, segment in ipairs(segments) do

      local segmentColours = segmentColourList[i]
      love.graphics.setCanvas(unpack(segment))
      love.graphics.clear(unpack(segmentColours))
    end
    
  end

  love.graphics.setCanvas()
end




function passes:applyLighting()
  
  local activeShader 
  
  if settings.graphics.fastLighting then
    activeShader = shader.lambertLightingSimple
  else
    activeShader = shader.lambertLighting
  end
  
  
  if camera.zoom*20 >= 2 then 
    local blurScale = (window.width/1920)
    blurCanvas(self.ambient, math.max(math.min(camera.zoom*10*blurScale, 30*blurScale), 15*blurScale) )
  end
  
  if not settings.graphics.fastLighting then
    activeShader:send("normalMap", self.normal)
  end
  activeShader:send("depthMap", self.depth)
  activeShader:send("ambientMap", self.ambient)
  
  love.graphics.setCanvas(self.scene)
  love.graphics.setShader(activeShader)
  
  love.graphics.draw(self.colour, 0, 0)
  
  
  love.graphics.setCanvas()
  love.graphics.setShader()
end


function passes:applyTimeBubble()
  
  
  
  
  if player1.timeBubble then

    love.graphics.setColor(1,1,1)

    local playerScreenX, playerScreenY = blue.standardTransform(player1.body:getX(), player1.body:getY())
    local cameraX, cameraY = blue.standardTransform(0, 0)

    shader.timeBubble:send("playerPos", {playerScreenX, playerScreenY})
    shader.timeBubble:send("cameraPos", {cameraX, cameraY})
    shader.timeBubble:send("zoom", camera:getFinalZoom())
    shader.timeBubble:send("time", love.timer.getTime())
    shader.timeBubble:send("roundAmount", smallTextureScale*camera.zoom)


  
    local tmp = self.final
    
    self.final = intermediate
    intermediate = tmp

    
    
    love.graphics.setShader(shader.timeBubble)
    love.graphics.setCanvas(self.final)
    
    
    love.graphics.draw(intermediate)
    
    
    love.graphics.setCanvas()
    love.graphics.setShader()
  
  end
  
  
end


function passes:applyBloom()
  
  --Bloom code goes here
  
end






function passes:combineCanvases()
  love.graphics.setBlendMode("alpha", "premultiplied")
  love.graphics.setColor(1,1,1)
  self:applyLighting()
  love.graphics.setBlendMode("alpha")
  self:applyBackground()
  
  self:applyTimeBubble()

end


function passes:applyBackground()
  love.graphics.setCanvas(self.final)
  love.graphics.clear(skyColour)
  for i, cloud in ipairs(cloudTable) do
    cloud:draw()
  end

  love.graphics.setColor(1,1,1)
  love.graphics.draw(self.scene)
  love.graphics.setCanvas()
end



function passes:render(x, y)
  lightHandler:send()
  x = x or 0
  y = y or 0
  self:combineCanvases()
  love.graphics.setColor(1,1,1)
  love.graphics.draw(self.final, x, y)
  --love.graphics.draw(self.ambient, x, y)
  self:clearCanvases()
end

return passes
