local controlHandler = {}

function controlHandler:getMoveDirection(controllerI)
  
  controllerI = controllerI or 0
  
  -- 0 = keyboard
  
  local direction = vector(0, 0)
  
  if controllerI == 0 then
  
    
    if love.keyboard.isScancodeDown("w") then
      direction = direction + vector(0, -1)
    end
    if love.keyboard.isScancodeDown("s") then
      direction = direction + vector(0, 1)
    end
    
    if love.keyboard.isScancodeDown("a") then
      direction = direction + vector(-1, 0)
    end
    if love.keyboard.isScancodeDown("d") then
      direction = direction + vector(1, 0)
    end
  
  else
    
    local controller = love.joystick.getJoysticks()[controllerI]
    
    if controller then
      direction = vector(controller:getGamepadAxis("leftx"), controller:getGamepadAxis("lefty"))*1.3

      if direction:magSq() < 0.5*0.5 then
        direction = vector(0,0)
      end
    end
  end
  
  
  if direction:magSq() > 1 then
    return direction:norm()
  else
    return direction
  end
  
end


function controlHandler:getAimDirection(controllerI, isoCorrect)
  
  controllerI = controllerI or 0
  
  -- 0 = keyboard
  
  local direction = vector(0, 0)
  
  if controllerI == 0 then
  
    local mv = vector(love.mouse.getPosition())
    
    local cv = vector(love.graphics.getWidth(), love.graphics.getHeight())*0.5
    
    direction = mv - cv
    
    
  else

    local controller = love.joystick.getJoysticks()[controllerI]
    
    if controller then
    
      direction = vector(controller:getGamepadAxis("rightx"), controller:getGamepadAxis("righty"))

      
      if direction:magSq() < 0.5*0.5 then
        direction = vector(0,0)
      end
    end
  end
  
  return direction:norm()

end



function controlHandler:sprintDown(controllerI)
  
  controllerI = controllerI or 0
  
  if controllerI == 0 then
    return love.keyboard.isScancodeDown("lshift")
  end

end



local mouseCallbacks = {}
local keyboardCallbacks = {}
local mousemovedCallbacks = {}
local mousereleasedCallbacks = {}
local wheelmovedCallbacks = {}

function controlHandler:subscribeMousepressed(entity, func)
  if mouseCallbacks[entity] then
    table.insert(mouseCallbacks[entity], func)
  else
    mouseCallbacks[entity] = {func}
  end
end

function controlHandler:unsubscribeMousepressed(entity)
  mouseCallbacks[entity] = nil
  --I can't imagine a scenario when you want to keep an entity subscribed but not a specific function
end



function controlHandler:subscribeKeypressed(entity, func)
  if keyboardCallbacks[entity] then
    table.insert(keyboardCallbacks[entity], func)
  else
    keyboardCallbacks[entity] = {func}
  end
end

function controlHandler:unsubscribeKeypressed(entity)
  keyboardCallbacks[entity] = nil
end


function controlHandler:subscribeMousemoved(entity, func)
  if mousemovedCallbacks[entity] then
    table.insert(mousemovedCallbacks[entity], func)
  else
    mousemovedCallbacks[entity] = {func}
  end
end

function controlHandler:unsubscribeMousemoved(entity)
  mousemovedCallbacks[entity] = nil
end


function controlHandler:subscribeMousereleased(entity, func)
  if mousereleasedCallbacks[entity] then
    table.insert(mousereleasedCallbacks[entity], func)
  else
    mousereleasedCallbacks[entity] = {func}
  end
end

function controlHandler:unsubscribeMousereleased(entity)
  mousereleasedCallbacks[entity] = nil
end

function controlHandler:subscribeWheelmoved(entity, func)
  local callTable = wheelmovedCallbacks
  if callTable[entity] then
    table.insert(callTable[entity], func)
  else
    callTable[entity] = {func}
  end
end

function controlHandler:unsubscribeWheelmoved(entity)
  wheelmovedCallbacks[entity] = nil
end


function love.mousepressed(...)
  
  for entity, functions in pairs(mouseCallbacks) do
    for i, func in ipairs(functions) do
      func(entity, ...)
    end
  end

end


function love.mousereleased(...)
  
  for entity, functions in pairs(mousereleasedCallbacks) do
    for i, func in ipairs(functions) do
      func(entity, ...)
    end
  end

end


function love.keypressed(...)
  
  for entity, functions in pairs(keyboardCallbacks) do
    for i, func in ipairs(functions) do
      func(entity, ...)
    end
  end
  
end


function love.mousemoved(...)
  
  for entity, functions in pairs(mousemovedCallbacks) do
    for i, func in ipairs(functions) do
      func(entity, ...)
    end
  end
  
end 


function love.wheelmoved(...)
  for entity, functions in pairs(wheelmovedCallbacks) do
    for i, func in ipairs(functions) do
      func(entity, ...)
    end
  end
  
end



return controlHandler